% It takes a base file called base.param
% and generates from that all the other files necesaary for a
% simulation
% Simulation takes place in two parts.
% First the initial conditions are generated in the
% box1.param box.geom  Geometry and box file for settling balls
% 
% This file takes an approximate number of balls and scales
% and generates all the files for a complete simulation
% First the appropriate radius is chosen

% up to here are inputs
real_num=350000;  % We're trying to simulate this many balls
simul_num = 1000;   % We're only going to use this many balls

sep = 10;         % How far down the slope the box is
box = [3.6 2.7 1.8]; % Dimensions of the box

base_name = '/home/jim/matlab/dem/base.param';
out_name  = 'full_sim'; % Name for output files
g1_name   = [out_name '1.geom'];
g2_name   = [out_name '2.geom'];
p1_name   = ['/home/jim/dem/data/' out_name '1.param'];
p2_name   = ['/home/jim/dem/data/' out_name '2.param'];
r1_name   = [out_name '1'];
r2_name   = [out_name '2'];


x = ping_pong({'radius' 'e_gravity' 'mass'}); % Get the particle properties
radius = x(1);
gravity = x(2);
mass = x(3);

mass = real_num/simul_num * mass;
radius = (real_num/simul_num)^(1/3) * radius;

pd = round(box*(simul_num/prod(box))^(1/3)); % Particle distribution
if any(2*radius*pd>box) % If any of the balls don't fit in the box
  pd(1:2) = floor(box(1:2)/(2*radius));
  pd(3) = round(simul_num/prod(pd(1:2)));
end
simul_num = prod(pd);
if simul_num*(4*radius^3) > prod(box)
  error('The balls can''t fit in the box')
end

% M file to calculate the box position and angle
% specifies 

hd = box/2;

x1 = -(sep+hd(1));  % X positions for box
x2 = -(sep-hd(1));  % X positions for box



[curve,width] = miyanomori;    % get curve information
[z dz] = plot_curve(curve,x1); % Calculate box position
z1 = z;
theta = atan(dz); % Angle of box
ct = cos(theta);
st = sin(theta);
xs = [x1   x1+box(1)*ct   x1+box(1)*ct-box(3)*st   x1-box(3)*st];
zs = [z1   z1+box(1)*st   z1+box(1)*st+box(3)*ct   z1+box(3)*ct];
mx = mean(xs);
mz = mean(zs);
clf
plot(xs,zs)
hold on
x = -sep+(-50:0.1:50);
[z dz] = plot_curve(curve,x); %
plot(x,z)
figure
% Make the geometry files
make_cuboid_geom(box,radius,[1 1 1;1 1 0],inf,g1_name);
make_geom('miyanomori',radius,[-160 max(xs)+5],g2_name);
delete(gcf)

% Now read in the input data files and make the output
fid = fopen(base_name,'r');
program_param = read_all_name_list(fid,'PROGRAM_PARAM');
record_param = read_all_name_list(fid,'RECORD_PARAM');
particle_param = read_all_name_list(fid,'PARTICLE_PARAM');
pp_param = read_all_name_list(fid,'COLLISION_PARAM');
pw_param = read_all_name_list(fid,'COLLISION_PARAM');
air_param = read_all_name_list(fid,'AIR_PARAM');
termination_param = read_all_name_list(fid,'TERMINATION_PARAM');
particle_state = read_all_name_list(fid,'PARTICLE_STATE');
fclose(fid);


disp(['Writing parameter file ' p1_name]);
fid = fopen(p1_name,'w');

% Choose special contact numbers to hasten equilibrium
pp1 = pp_param;
pw1 = pw_param;
pp1.friction = 0;
pp1.restitution = 0.1;
pw1.friction = 0;
pw1.restitution  = 0.1;
program_param.geom_file_name = g1_name;
record1.rec_name = r1_name;
record1.rec_position = 'T';
record1.rec_t = 0.05;
particle_param.p_radius = radius;
particle_param.p_mass = mass;
particle_param.p_gravity = -gravity * [st 0 ct];
term1.max_t = 5;
init1.origin = [0 0 pd(3)*radius-box(3)/2];
init1.initial_type = 'cuboid';
init1.cube_size    = pd;
init1.particle_num = simul_num;
init1.random_lin_vel = 'T';
init1.lin_vel_max = 5;



write_name_list(fid,program_param,'PROGRAM_PARAM');
write_name_list(fid,record1,'RECORD_PARAM');
write_name_list(fid,particle_param,'PARTICLE_PARAM');
write_name_list(fid,pp1,'COLLISION_PARAM');
write_name_list(fid,pw1,'COLLISION_PARAM');
write_name_list(fid,air_param,'AIR_PARAM');
write_name_list(fid,term1,'TERMINATION_PARAM');
write_name_list(fid,init1,'PARTICLE_STATE');
fclose(fid);


program_param.geom_file_name = g2_name;
record_param.rec_name = r2_name;
particle_param.p_gravity = [0 0 -gravity];
init2.initial_type    = 'data';
init2.file_name       = [r1_name '.dump'];
init2.origin          = [mx 0 mz];
init2.rotation = [0 -theta 0];
init2.particle_num = simul_num;

disp(['Writing parameter file ' p2_name]);
fid = fopen(p2_name,'w');
write_name_list(fid,program_param,'PROGRAM_PARAM');
write_name_list(fid,record_param,'RECORD_PARAM');
write_name_list(fid,particle_param,'PARTICLE_PARAM');
write_name_list(fid,pp_param,'COLLISION_PARAM');
write_name_list(fid,pw_param,'COLLISION_PARAM');
write_name_list(fid,air_param,'AIR_PARAM');
write_name_list(fid,termination_param,'TERMINATION_PARAM');
write_name_list(fid,init2,'PARTICLE_STATE');
fclose(fid);
