function  [y,dy] = calc_curve(c,x)
% Plot the data in the curve c over it's range
% or at points specified by x

SRT = 1; % Straight line
ARC = 2; % Circular arc
CBS = 3; % Cubic spline

for i=1:length(c)
  cxp(i) = c(i).p(1);
  cxq(i) = c(i).q(1);
end

if nargin<2
  npoints = 1000;
  minx = min([cxp cxq]);
  maxx = max([cxp cxq]);
  x = minx:(maxx-minx)/(npoints-1):maxx;
end
y  = repmat(NaN,[length(c) length(x)]);
dy = repmat(NaN,[length(c) length(x)]);

for h=1:length(c) % Step through each curve section calculating 
  p = c(h).p;
  q = c(h).q;
  dp = c(h).dp;
  f = find((x>=c(h).p(1)) & (x< c(h).q(1)));
  switch c(h).type
  case SRT, % Straight line
    if dp(1) ~= 0
      g = dp(2)/dp(1);
    else
      g = sign(dp(2))*Inf;
    end
    y(h,f) = p(2)- g*p(1)+g*x(f);
    dy(h,f) = repmat(g,[1 length(f)]); 
  case ARC,             % A circular arc of specified radius
    r = c(h).r;
    a = c(h).c;
    y(h,f)  = a(2) - r*sqrt(1 - (x(f)-a(1)).^2/r^2);
    dy(h,f) = (x(f)-a(1))./(r*sqrt(1 - (x(f)-a(1)).^2/r^2));
 case CBS,
  y1d = dx1/2*tan(an(1,h)/360*2*pi); % Calc the gradients
    y2d = dx1/2*tan(an(2,h)/360*2*pi);
    a = (y1+y2)/2-(y2d-y1d)/4;
    b = 3*(y2-y1)/4-(y1d+y2d)/4;
    c = (y2d-y1d)/4;
    d = (y1d+y2d)/4 + (y1-y2)/4;
    %if abs(a+b+c+d - y2)>eps error; end
    %if abs(a-b+c-d - y1)>eps error; end
    %if abs(b+2*c+3*d - y2d)>100*eps error; end
    %if abs(b-2*c+3*d - y1d)>100*eps error; end
    x = (2*x-(x1+x2))/dx1;
    y = a + x.*(b + x.*(c+x*d)); 
    dy = 2/dx1*(b + x.*(2*c+x*3*d));
    ddy = 2/dx1*2/dx1*(2*c+x*6*d);
    r=(1+dy^2)^3/2/ddy;
   end
end

if nargout==0  % If there are no ouput arguments plot the curve
  subplot(2,1,1)
  hold on
  plot(x,y)
  hold on
  subplot(2,1,2)
  plot(x,dy)
  y = [];dy=[];
end
y=y(~(isnan(y)));
dy=dy(~(isnan(dy)));


