function display_dem(fname,sz,times,type,camera_move,bounding_box,output)
% fname name ofdata file
% sz dimension in  pixels of figure or if vector sqrt of total number of pixels
% step skips frames =1 means display every frame
% type a number specifying how much detail the balls are to be drawn with
% camera_move - move the camera with the balls
% bounding_box - display a bounding box in the corner
% makempeg - make an mpeg movie
% Various kinds of display are possible
% A picture of the whole geometry and all the balls
% A close up of only around the balls
% Both combined together
% For example
% output can be jpeg, tiff, mpeg, postscript
% If mpeg chooses jpeg and makes mpeg movie
% If the size is zero makes no pictures just displays
% 451 x 533  a good aspect ratio
% 614 x 711
% display_dem('kouichi_paperb',[700 780],[],20,1,1,'mpeg')

if nargin < 1
  fname='test';                      % Name for data file and final mpeg file
elseif isempty(fname)
  fname='test';                      % Name for data file and final mpeg file
end   

if nargin < 2                      % Sixe of jpeg pictures
  sz = [400 400];
elseif isempty(sz)
  sz = [400 400];
end

if nargin < 3                      % plot every step frames
  times = [];                      % Plot all times
end

if nargin < 4                   
  type = [];                    % Auto selection of sphere resolution
end

if nargin < 5                    % Move the camera with the balls
 camera_move = 1;
elseif isempty(camera_move)
 camera_move = 1;
end

if nargin < 6                  
 bounding_box = 0;               % Plot a bounding  box as well
elseif isempty(bounding_box)
 bounding_box = 0;
end  

if nargin < 7                  
 output = 'mpeg'
elseif isempty(bounding_box)
 output = 'mpeg'
end  

tiff = 0;
jpeg = 0;
eps  = 0;
mpeg = 0;
if prod(sz)>0; % Does the picture have any size
  switch output
    case 'mpeg'
      jpeg = 1;
      mpeg = 1;
    case 'jpeg'
      jpeg = 1;
    case 'tiff';
      tiff = 1;
    case 'eps';
      eps = 1;
    otherwise
      error(['Unknown output requested' output'])
  end
end

rootdir = '/home/jim/dem/';   % Root directory
% Directory structure % Input directories
inputdirs = {[rootdir 'data/'], [rootdir 'testfiles/'], [rootdir 'rotating/']};
outputname = '/tmp/tmpdem.';   % Root name for temporary image files
mpegparamin = [rootdir 'mpeg.param.in'];
mpegparam   = [rootdir 'mpeg.param'];

disp('Reading data files')
% Get physical paramters 
fid_p = fopen(filename([fname '.param'],inputdirs),'r');
s = read_all_name_list(fid_p,'PROGRAM_PARAM');
geom_name = s.geom_file_name; disp(['Geom name ' geom_name]);
s = read_all_name_list(fid_p,'RECORD_PARAM');
pname = filename([s.rec_name '.position'],inputdirs);
mpegname = [rootdir 'mpeg/' s.rec_name '.mpg']; 
disp(['Position data name ' pname]);
s = read_all_name_list(fid_p,'PARTICLE_PARAM');
radius = s.p_radius; disp(['radius ' num2str(radius)])
read_all_name_list(fid_p,'COLLISION_PARAM');
read_all_name_list(fid_p,'COLLISION_PARAM');
read_all_name_list(fid_p,'AIR_PARAM');
read_all_name_list(fid_p,'TERMINATION_PARAM');
s = read_all_name_list(fid_p,'PARTICLE_STATE');
num_p=s.particle_num;disp(['Number of particles ' num2str(num_p)]);
fclose(fid_p);
% Read all the geometry stuff 
[bd,gname,gbounds,gradius] = read_geometry(filename(geom_name,inputdirs));

cylinder = 0;
miyanomori=0;
if length(findstr('miyanomori',gname))
  miyanomori=1;
elseif length(findstr('cylinder',gname))
  cylinder=1;
elseif length(findstr('box',gname))
  box=1;
end
colors = rand(num_p,3); % ball colours random

fid = fopen(pname,'r');

clf; % Clear the current figure

set(gcf,'defaultpatchfacelighting','flat');
set(gcf,'defaultpatchedgecolor','none');
set(gcf,'defaultpatchbackfacelighting','lit');

set(gcf,'defaultsurfacefacelighting','flat');
set(gcf,'defaultsurfaceedgecolor','none');
set(gcf,'defaultsurfacebackfacelighting','lit');
set(gcf,'defaultsurfacefacecolor','texturemap');

lw = sqrt(prod(sz))/400;
set(gcf,'defaultlinelinewidth',lw)
set(gcf,'defaultpatchlinewidth',lw)
set(gcf,'defaultsurfacelinewidth',lw)
set(gcf,'defaultaxeslinewidth',lw)

set(gcf,'defaulttextfontsize',max(sqrt(prod(sz))/40,8))

ta = axes('position',[0 0 1 1],'visible','off'); % Create axis for text
% Create handles for text
str1 = text(0,1,'step:999999','HorizontalAlignment','left',...
    'VerticalAlignment','top','fontweight','bold');
str2 = text(1,1,'time:999999','HorizontalAlignment','right',...
    'VerticalAlignment','top','fontweight','bold');


% set up axis for balls
ba = axes('position',[0 0 1 1]); % Create axis for balls
bhdls = []; % Handles for balls % None yet
if miyanomori == 1
  [mx,my,mz] = plot_miyanomori([],4);% 
  ax = [min(mx(:)) max(mx(:)) min(my(:)) max(my(:)) min(mz(:)) max(mz(:))];
  axis(ax);
  h = mesh(mx,my,mz); % plot the background
  set(h,'linewidth',lw)
  %surfl(mx,my,mz) % plot the background
else
  axis(gbounds([1 4 2 5 3 6])+[-radius radius -radius radius -radius radius]')
end
if miyanomori == 1
else
  %   plot_cylinder(w_length,w_radius);
end
axis equal
axis off
axis manual      % Freeze Axes limits
light('position',[-3 1 2])  
material shiny  
hold on

% Create axis for inline ellipse
if bounding_box
  ia = axes('position',[0 0.3 .65 .95]); 
  ellpsh = [];
  if miyanomori == 1
    [mx,my,mz] = plot_miyanomori([],6);% 
    h = mesh(mx,my,mz); % plot the background
    set(h,'linewidth',lw/4)
  else
    %   plot_cylinder(w_length,w_radius);
  end
  axis equal
  axis off
  axis manual      % Freeze Axes limits
  light('position',[-3 1 2])  
  material shiny  
  hold on
end

if isempty(type)
  ts = [20 15 10 5 4 3 2 2]
  type = ts(min([7,floor(num_p^(1/3))]))
end

if     type == 1 disp('Using markers')
elseif type == 2 disp('Using tetrahedrons')
elseif type == 3 disp('Using cubes')
elseif type == 4 disp('Using octahedrons')
elseif type >= 5 disp(['Using ' num2str(type) ' point spheres'])
end
 
if isempty(times)
  disp(['Displaying at all times'])
else
  disp(['Displaying at times ' num2str(times)])
end

% Create the individual frames in Jpeg format
disp('Processing pictures')
k = 0; % Count of pictures processed
lt = - cputime;
lasttime = -inf;
k = 0;
times = sort(times);

while(~feof(fid))
  [r time cstep] = decode_dem(fid,1); % Read one time period
  draw = 1;
  if length(times) > 0
    if k+1 > length(times)
      break
    end
    if (times(k+1) <= lasttime) | (times(k+1) > time)
      lasttime = time;
      draw = 0;
    end
  end

  if draw == 1
  
  k = k + 1; % Sucessfully read new positions 
  %disp(['Data for time ' num2str(time) ' read'])
  axes(ba)  % Ball axis
  delete(bhdls);
  bhdls = mysphere(r,radius,colors,type);

  if (camera_move==1)
    minp = min(r,[],2);
    maxp = max(r,[],2);
    rg = max(maxp-minp)/2+1;
    mn = (minp+maxp)/2;
    axis(reshape([mn-rg mn+rg]',1,6))  % Choose square axis
  end

  if bounding_box
    axes(ia)
    delete(ellpsh)
    ellpsh = inertia_ellipsoid(r);
  end
  
  axes(ta)
  set(str1,'string',['step: ' num2str(cstep)]);
  set(str2,'string',sprintf('time: %3.2f',time));
  %writeppm([outputname num2str(k) '.ppm'],sz);
  %writejpg([outputname num2str(k) '.jpg'],sz);
  if (jpeg==1) printjpg([outputname num2str(k) '.jpg'],sz);end
  if (tiff==1) printtiff([outputname num2str(k) '.tiff'],sz);end
  if (eps==1) printeps([outputname num2str(k) '.eps'],sz);end
  drawnow
  %disp(['Time ' num2str(time) ' finished'])
%  if step>1
%    decode_dem(fid,step-1);
%  end
%  k = k + step-1; % Sucessfully read new positions 
end
end
fclose(fid);
lt = (lt + cputime)/k;
disp([num2str(lt) ' seconds a frame'])
if ~mpeg
  return
end

  % Now create the mpeg movie
  % Crashes with jpeg input so convert to ppm
  unix(['cp ' mpegparamin ' ' mpegparam]);
  fid = fopen(mpegparam,'a');
  fprintf(fid,'\n#Auto generated\n');
  fprintf(fid,['INPUT_CONVERT convert * ppm:-\n']);
  fprintf(fid,['BASE_FILE_FORMAT PPM\n']);
  fprintf(fid,['OUTPUT ' mpegname '\n']);  
  fprintf(fid,['INPUT_DIR \n']);
  fprintf(fid,'INPUT\n');
  if (jpeg==1) fprintf(fid,[outputname '*.jpg [1-' num2str(k) '+1]\n'])
  end
  if (tiff==1)
    fprintf(fid,[outputname '*.tif [1-' num2str(k) '+1]\n'])
  end
  fprintf(fid,'END_INPUT\n');
  fclose(fid);
  %unix(['mpeg_encode ' mpegparam ' > /dev/null']); % Encode mpeg
  unix(['mpeg_encode -quiet -1 -no_frame_summary ' mpegparam]); % Encode mpeg
  %unix(['mtv 'mpegname ' &']); % Spawn an mpeg  player 


% $$$   cstep=fread(fid,1,'float32');
% $$$   time=fread(fid,1,'float32');
% $$$   energy=fread(fid,1,'float32');
% $$$   [r count]=fread(fid,[3 num_p],'float32');
