function display_dem2d(fname,sz,times,type,camera_move,ellipsoid,output,dims,colour_time)
% fname name ofdata file
% sz dimension in  pixels of figure or if vector sqrt of total number of pixels
% step skips frames =1 means display every frame
% type a number specifying how much detail the balls are to be drawn with
% camera_move = 0 Fix according to geometry
%               1 move the camera with the balls
%               2 increase axis with balls
% ellipsoid - display a bounding box in the corner
% makempeg - make an mpeg movie
% Various kinds of display are possible
% A picture of the whole geometry and all the balls
% A close up of only around the balls
% Both combined together
% For example
% output can be jpeg, tiff, mpeg, postscript
% If mpeg chooses jpeg and makes mpeg movie
% If the size is zero makes no pictures just displays
% 451 x 533  a good aspect ratio
% 614 x 711
% display_dem('kouichi_paperb',[700 780],[],20,1,1,'mpeg')

if nargin < 1; fname = [];end
if nargin < 2; sz = [];end
if nargin < 3; times = [];end
if nargin < 4; type = [];end
if nargin < 5; camera_move = [];end
if nargin < 6; ellipsoid = [];end
if nargin < 7; output = [];end
if nargin < 8; dims = [];end
if nargin < 9; colour_time = [];end

if isempty(fname); fname='test'; end
if isempty(sz); sz = [];end
if isempty(times); times= [];end
if isempty(type); type=1;end
if isempty(camera_move); camera_move = 0;end
if isempty(ellipsoid); ellipsoid = 0; end
if isempty(output);output = 'mpeg';end
if isempty(dims);dims = 2;end
if isempty(colour_time);colour_time = -NaN;end

if ~isempty(sz); disp(['Size ' num2str(sz)]);end
if isempty(times)
  disp(['Displaying at all times']);
else
  disp(['Displaying at times ' num2str(times)]);
end
disp(['Circle type ' num2str(type)]);
if camera_move==1; disp('Moving Camera'); end
if camera_move==1; disp('Increaseing axis'); end
if ellipsoid==1; disp('Plotting momentum ellipsoid'); end
disp(['Output ' output]);
disp (['Dimensions ' num2str(dims)]);  
disp(['Colour_time ' num2str(colour_time)]);


tiff = 0;
jpeg = 0;
eps  = 0;
mpeg = 0;
if ~isempty(sz) % Does the picture have any size
  switch output
  case 'mpeg'
    jpeg = 1;
    mpeg = 1;
  case 'jpeg'
    jpeg = 1;
  case 'tiff';
      tiff = 1;
  case 'eps';
      eps = 1;
  otherwise
      error(['Unknown output requested' output'])
  end
end

rootdir = '/home/jim/dem/';   % Root directory
% Input directories

inputdirs = {rootdir, [rootdir 'data/'], [rootdir 'testfiles/'], ...
      [rootdir '2d/'], [rootdir 'rotating/'] };
  
outputname = '/tmp/tmpdem.';   % Root name for temporary image files
mpegparamin = [rootdir 'mpeg.param.in'];
mpegparam   = [rootdir 'mpeg.param'];

% Find the directory where the param file is
param_name = filename([fname '.param'],inputdirs);
dirname = param_name(1:max(find(param_name=='/')));

disp(['Reading data files from ' param_name])
s = read_name_lists(param_name);
geom_name = [dirname s.program_param.geom_file_name]; 
disp(['Geom name ' geom_name]);
pname = [dirname s.record_param.rec_name '.position'];
ppname = [dirname s.record_param.rec_name '.pprops'];
mpegname = [rootdir 'mpeg/' s.record_param.rec_name '.mpg']; 
disp(['Position data name ' pname]);
in_radii = s.particle_param.p_radius(1:s.particle_param.p_num)'; 
disp(['Input radii ' num2str(in_radii)])
num_p=s.particle_state.particle_num;
disp(['Number of particles ' num2str(num_p)]);

if length(in_radii)==1
  radius = repmat(in_radii(1),[num_p 1]);
else
  fid_p = fopen(ppname,'r');
  radius = read_part_props(fid_p,num_p);
  fclose(fid_p);
end

colours = [];

fid = fopen(pname,'r');

clf; % Clear the current figure

lw = max(sqrt(prod(sz))/400,1);
set(gcf,'defaultlinelinewidth',lw)
set(gcf,'defaultpatchlinewidth',lw)
set(gcf,'defaultsurfacelinewidth',lw)
set(gcf,'defaultaxeslinewidth',lw)
set(gcf,'defaultlinemarkeredgecolor','none')
set(gcf,'defaulttextfontsize',max(sqrt(prod(sz))/40,8))

ta = axes('position',[0.01 0.01 .98 .98],'visible','off'); % Create axis for text
ba = axes('position',[0.01 .01  .98  .98]);    % Create axis for balls
ia = axes('position',[0.01 .735 .245 .245]);   % Create axis for inertia
indent_axis(2);

% Create handles for text
axes(ta);
str1 = text(0,1,'step:999999','HorizontalAlignment','left',...
    'VerticalAlignment','top','fontweight','bold');
str2 = text(1,1,'time:999999','HorizontalAlignment','right',...
    'VerticalAlignment','top','fontweight','bold');

% Nasty kludges to make figures print well
fig_u = get(gcf,'units');
set(gcf,'units','pixels');
fig_sz = get(gcf,'position');   %Size of figure in pixels
set(gcf,'units',fig_u);

if (jpeg==1 | tiff == 1| eps ==1)
  if (type==1)
    radius = radius/2;
  end
  set(gcf,'units','pixels');
  fig_sz = [fig_sz(1:2) sz];
  set(gcf,'position',fig_sz);
  set(gcf,'units',fig_u);
end

% Create axis for balls
bhdls = []; % Handles for balls % None yet

max_radius = max(radius);
min_radius = min(radius);
disp(['Max radius ' num2str(max_radius) ', min radius ' num2str(min_radius)]);

axes(ba);
fid_g = fopen(geom_name,'r');
display_geom(fid_g,dims); % Display  geometry
fclose(fid_g);
axis equal
axis off
axis manual      % Freeze Axes limits
hold on

% Create axis for inline ellipse
if ellipsoid
  axes(ia);
  fid_g = fopen(geom_name,'r');
  display_geom(fid_g,dims); % Display  geometry
  fclose(fid_g);
  ellpsh = [];
  axis equal
  axis off
  axis manual      % Freeze Axes limitounding_boxs
  hold on
else
  delete(ia)
end

minp = [0;0];
maxp = [0;0];

% Create the individual frames in Jpeg format
disp('Processing pictures')
k = 0; % Count of pictures processed
lt = - cputime;
lasttime = -inf;
k = 0;
times = sort(times);
while(~feof(fid))
  [r time cstep] = decode_dem(fid,1,dims); % Read one time period
  if size(r,1)==3
    r(3,:) = [];
  end
  draw = 1;
  if length(times) > 0
    if k+1 > length(times)
      break
    end
    if (times(k+1) <= lasttime) | (times(k+1) > time)
      lasttime = time;
      draw = 0;
    end
  end
  if length(colours) == 0 | time < colour_time
    ca = 2*atan2(r(1,:),r(2,:)); % Get colour angle
    max_r = sqrt(max(sum(r.*r,1)));
    lum = .5+.5*sqrt(sum(r.*r,1))/max_r;
    colours = .5*(repmat(lum,[3 1]).*[1+sin(ca+pi/3); 1+sin(ca-pi/3); 1+sin(ca+pi)])';
    xrel = abs(r(2,:))/max_r;
    f = find(xrel<.05);
    colours(f,:) = repmat([1 1 1],[length(f) 1]);
    f = find(.05<=xrel  & xrel < .15);
    colours(f,:) = repmat([0 0 0],[length(f) 1]);
  end
  if draw == 1
    
    k = k + 1; % Sucessfully read new positions 
    %disp(['Data for time ' num2str(time) ' read'])
    axes(ba)  % Ball axis
    delete(bhdls);
    bhdls = mycircle(r,radius,colours,type);
    
    if (camera_move==1)
      minp = min(r,[],2)-max_radius;
      maxp = max(r,[],2)+max_radius;
      axis(reshape([minp maxp]',1,4))  % Choose square axis
    elseif(camera_move==2)
      minp = min([r minp],[],2);
      maxp = max([r maxp],[],2);
      axis(reshape([minp-max_radius maxp+max_radius]',1,4))  % Choose square axis
    elseif(camera_move==3)
      minp = .9*minp+.1*min(r,[],2);
      maxp = .9*maxp+.1*max(r,[],2);
      minp = min([r minp],[],2);
      maxp = max([r maxp],[],2);
      axis(reshape([minp-max_radius maxp+max_radius]',1,4))  % Choose square axis
    end
    
    if ellipsoid
      axes(ia)
      delete(ellpsh)
      ellpsh = inertia_ellipsoid(r);
    end
    
    axes(ta)
    set(str1,'string',['step: ' num2str(cstep)]);
    set(str2,'string',sprintf('time: %3.2f',time));
    %writeppm([outputname num2str(k) '.ppm'],sz);
    %writejpg([outputname num2str(k) '.jpg'],sz);
    if (jpeg==1) 
      printjpg([outputname num2str(k) '.jpg'],sz);
    elseif (tiff==1) 
      printtiff([outputname num2str(k) '.tiff'],sz);
    elseif (eps==1) 
      printeps([outputname num2str(k) '.eps'],sz);
    else 
      drawnow
    end
  end
end
fclose(fid);
lt = (lt + cputime)/k;
disp([num2str(lt) ' seconds a frame'])
if ~mpeg
  return
end

  % Now create the mpeg movie
  % Crashes with jpeg input so convert to ppm
  unix(['cp ' mpegparamin ' ' mpegparam]);
  fid = fopen(mpegparam,'a');
  fprintf(fid,'\n#Auto generated\n');
  fprintf(fid,['INPUT_CONVERT convert * ppm:-\n']);
  fprintf(fid,['BASE_FILE_FORMAT PPM\n']);
  fprintf(fid,['OUTPUT ' mpegname '\n']);  
  fprintf(fid,['INPUT_DIR \n']);
  fprintf(fid,'INPUT\n');
  if (jpeg==1) fprintf(fid,[outputname '*.jpg [1-' num2str(k) '+1]\n'])
  end
  if (tiff==1)
    fprintf(fid,[outputname '*.tif [1-' num2str(k) '+1]\n'])
  end
  fprintf(fid,'END_INPUT\n');
  fclose(fid);
  %unix(['mpeg_encode ' mpegparam ' > /dev/null']); % Encode mpeg
  unix(['mpeg_encode -quiet -1 -no_frame_summary ' mpegparam]); % Encode mpeg
  %unix(['mtv 'mpegname ' &']); % Spawn an mpeg  player 


% $$$   cstep=fread(fid,1,'float32');
% $$$   time=fread(fid,1,'float32');
% $$$   energy=fread(fid,1,'float32');
% $$$   [r count]=fread(fid,[3 num_p],'float32');
