function display_energy(fname, option, limit)
% Diplayinformation from a dem energy file
% fname name of energy data file 
% option = 1 Plot all on one graph
% option = 2 Plot k.e, p.e and total energy graphs
% option = 3 Plot total k.e., plus p.e, plus total
% option = 4 Plot total k.e., plus g.p.e, plus c.p.e plus total
% option = 5 Assumes total energy is exponentially decaying and
% does a log plot against the minimum
% Limit psecifies what range of values to display
if nargin < 3
  limit = 0;
end
if nargin < 2
  option = 1;
end
if nargin < 1
  fname='test';              % Name for base file
end

lke = 1;
ake = 2;
gpe = 3;
cpe = 4;
tke = 5;
tpe = 6;
te = 7;
tens = 7;


% Directory structure
inputdir = '/home/jim/dem/data';% Input directory for dem data files
olddir = pwd;
cd(inputdir);
disp('Reading data files')
% Get physical paramters 
a = decodedem([fname '.param'],...
    {'PARTICLE_RADIUS' 'GEOM_CYLINDER_LENGTH' 'GEOM_CYLINDER_RADIUS'});
if any(isnan(a)) error(['Error scanning ' fname '.param file']);end
radius = a(1); w_length = a(2);w_radius = a(3);

% Get number of particles 
a = decodedem([fname '.init'],{'PARTICLE_NUM' 'PROGRAM_STEP' 'PROGRAM_TIME'});
if any(isnan(a)) error(['Error scanning ' fname '.init file']);end
num_p=a(1);

fid = fopen([fname '.energy'],'r');

e = fscanf(fid,'%f %f %f %f %f %f %f\n');
n = length(e(:))/tens;
e = reshape(e(:),tens,n);

if length(limit) == 2
  e = e(:,limit(1):limit(2));
  n = 1 + limit(2) - limit(1); 
end

errc = max(abs(e(:)))*20*sqrt(eps);

if(max(abs(e(lke,:) + e(ake,:)-e(tke,:)))>errc)
  disp('Kinetic energies do not sum')
  disp(max(abs(e(lke,:) + e(ake,:)-e(tke,:))))
end
if(max(abs(e(gpe,:) + e(cpe,:)-e(tpe,:)))>errc)
  disp('Potential energies do not sum')
  disp(max(abs(e(gpe,:) + e(cpe,:)-e(tpe,:))))
end
if(max(abs(e(tke,:) + e(tpe,:)-e(te,:)))>errc)
  disp('Energy total is wrong')
  disp(max(abs(e(tke,:) + e(tpe,:)-e(te,:))))
end
e(te,:) = e(te,:) - mean(e(te,:));

disp(['Max total energy change is ' num2str(max(e(te,:))-min(e(te,:)))])
delete(get(gcf,'children')); % Clear the current figure

if option == 1
  subplot(3,1,1)
  plot(1:n,e([lke  ake tke],:))
  legend('Linear k.e.', 'Angular k.e.',  'Total k.e.')
  subplot(3,1,2)
  plot(1:n,e([gpe cpe tpe],:))
  legend('Gravitational p.e.', 'Contact p.e',  'Total p.e.')
  subplot(3,1,3)
  plot(1:n,e(te,:))
  legend('Total energy')
elseif option == 2
  plot(1:n,e([lke ake gpe cpe tke tpe te],:))
  legend('Linear k.e.', 'Angular k.e.', 'Gravitational p.e.', ...
      'Contact p.e', 'Total k.e.', 'Total p.e.', 'Total energy')
elseif option == 3
  subplot(3,1,1)
  plot(1:n,e(tke,:))
  legend('Total k.e.')
  subplot(3,1,2)
  plot(1:n,e(tpe,:))
  legend('Total p.e.')
  subplot(3,1,3)
  plot(1:n,e(te,:))
  legend('Total energy')
elseif option == 4
  subplot(4,1,1)
  plot(1:n,e(tke,:))
  legend('Total k.e.')
  subplot(4,1,2)
  plot(1:n,e(gpe,:))
  legend('Gravitational p.e.')
  subplot(4,1,3)
  plot(1:n,e(cpe,:))
  legend('Contact p.e.')
  subplot(4,1,4)
  plot(1:n,e(te,:))
  legend('Total energy')
elseif option == 5
  e = e(te,:)-min(e(te,:))+max(e(te,:))*10*eps;
  plot(1:n,log(e))
  legend('Log plot of energy')
elseif option == 6
  subplot(3,1,1)
  plot(1:n,e(lke,:))
  legend('Linear k.e.')
  subplot(3,1,2)
  plot(1:n,e(ake,:))
  legend('Angular kinetic p.e.')
  subplot(3,1,3)
  plot(1:n,e(te,:))
  legend('Total energy')
end

