function dims = display_geom(fid,dims)

if nargin < 2
  dims == [];
end

gp = read_name_list(fid,'GEOMETRY_PARAM');
for i=1:gp.geom_num
  bp(i) = read_name_list(fid,'BOUNDARY_PARAM');
end

hold on
if nargin > 1
  gp.geom_dims = dims;
end
if ~isfield(gp,'geom_dims') 
  disp('Assuming geom_dim 2');
  gp.geom_dims = 2;
end
dims = gp.geom_dims;

if gp.geom_dims == 2
  if isfield(gp,'geom_bounds')
    xmin = gp.geom_bounds(1);
    ymin = gp.geom_bounds(2);
    xmax = gp.geom_bounds(3);
    ymax = gp.geom_bounds(4);
  else
    xmin = -10; ymin = -10; xmax = 10; ymax = 10;
  end
  xrg = xmax-xmin;
  yrg = ymax-ymin;
  rrg = sqrt(xrg^2 + yrg^2);
  for i=1:gp.geom_num
    switch bp(i).bd_type
      case {1,2} % x plane
      plot([bp(i).bd_s(1) bp(i).bd_s(1)], [ymin ymax])
    case {3,4} % y plane
      plot([xmin xmax],[bp(i).bd_s(1) bp(i).bd_s(1)])
    case {7}   % general plane
      tan_v = [-bp(i).v(2) bp(i).v(1)];
      xc = bp(i).bd_v(1)*bp(i).bd_s(1)+tan_v(1)*[-rrg rrg];
      xy = bp(i).bd_v(2)*bp(i).bd_s(1)+tan_v(2)*[-rrg rrg];
      plot(xc,yc)
    case {8,9} % circle
      p = 0:2*pi/200:2*pi;
      xc = bp(i).bd_s(2)*cos(p)+bp(i).bd_s(1)*bp(i).bd_v(1);
      yc = bp(i).bd_s(2)*sin(p)+bp(i).bd_s(1)*bp(i).bd_v(2);
      plot(xc,yc)
     case {10} % a circle of circles
      p = 0:2*pi/200:2*pi;
      xc = (bp(i).bd_s(2)+bp(i).bd_s(4))*cos(p)+bp(i).bd_s(1)*bp(i).bd_v(1);
      yc = (bp(i).bd_s(2)+bp(i).bd_s(4))*sin(p)+bp(i).bd_s(1)*bp(i).bd_v(2);
      plot(xc,yc)
      xc = (bp(i).bd_s(2)-bp(i).bd_s(4))*cos(p)+bp(i).bd_s(1)*bp(i).bd_v(1);
      yc = (bp(i).bd_s(2)-bp(i).bd_s(4))*sin(p)+bp(i).bd_s(1)*bp(i).bd_v(2);
     plot(xc,yc)
    otherwise
      error('Unknown boundary type')
    end
  end
  axis([xmin xmax ymin ymax])  
else
  if isfield(gp,'geom_bounds')
    xmin = gp.geom_bounds(1);
    ymin = gp.geom_bounds(2);
    zmin = gp.geom_bounds(3);
    xmax = gp.geom_bounds(4);
    ymax = gp.geom_bounds(5);
    zmax = gp.geom_bounds(6);
  else
    xmin = -10; ymin = -10; zmin = -10; xmax = 10; ymax = 10;; zmax = 10;
  end
  xrg = xmax-xmin;
  yrg = ymax-ymin;
  zrg = ymax-ymin;
  rrg = sqrt(xrg^2 + yrg^2 + zrg^2 );
  for i=1:gp.geom_num
    switch bp(i).bd_type
      case {1,2} % x plane
      plot([bp(i).bd_s(1) bp(i).bd_s(1)], [ymin ymax])
      keyboard
    case {3,4} % y plane
      plot([xmin xmax],[bp(i).bd_s(1) bp(i).bd_s(1)])
    case {5,6} % z plane
    case {7}   % general plane
      tan_v = [-bp(i).v(2) bp(i).v(1)];
      xc = bp(i).bd_v(1)*bp(i).bd_s(1)+tan_v(1)*[-rrg rrg];
      xy = bp(i).bd_v(2)*bp(i).bd_s(1)+tan_v(2)*[-rrg rrg];
      plot(xc,yc)
    case {8,9} % circle
      p = 0:2*pi/200:2*pi;
      xc = bp(i).bd_s(2)*cos(p)+bp(i).bd_s(1)*bp(i).bd_v(1);
      yc = bp(i).bd_s(2)*sin(p)+bp(i).bd_s(1)*bp(i).bd_v(2);
      plot(xc,yc)
     case {10} % a circle of circles
      p = 0:2*pi/200:2*pi;
      xc = (bp(i).bd_s(2)+bp(i).bd_s(4))*cos(p)+bp(i).bd_s(1)*bp(i).bd_v(1);
      yc = (bp(i).bd_s(2)+bp(i).bd_s(4))*sin(p)+bp(i).bd_s(1)*bp(i).bd_v(2);
      plot(xc,yc)
      xc = (bp(i).bd_s(2)-bp(i).bd_s(4))*cos(p)+bp(i).bd_s(1)*bp(i).bd_v(1);
      yc = (bp(i).bd_s(2)-bp(i).bd_s(4))*sin(p)+bp(i).bd_s(1)*bp(i).bd_v(2);
     plot(xc,yc)
    otherwise
      error('Unknown boundary type')
    end
  end
  axis([xmin xmax ymin ymax])  
end
