function  make_cuboid_geom(s,radius,open,t,gname)
% Create a cuboid geometry centred on the origin with specified
% s three dimensions for the box
% radius radius of balls the box is designed for
% open whether any sides of the box are to be open
% t thickness of the walls default is inf
% The top of the box is open
% make_cuboid_geom([2.7 3.6 1.8],.14,[1 1 1;1 1 0])
if nargin < 1
  s = [1 1 1];
end
if nargin < 2
  radius = 0;             % No radius adjustment
end
if nargin < 3
  open = ones(2,3);    % All sides are present
end
if nargin < 4
  t = inf;             % sides of the box are infinitely thick
end
if nargin < 5
  gname = 'box.geom';
end

s = s(:);

rg = [-s/2-radius;s/2+radius];  % region  of geometry plus a couple of radius

mn = -s/2-t-radius;    % Minimum
mx = s/2+t+radius;     % Maxmum
s = s/2-radius;                  % Adjust size of particle radius


boundary(1).v      = [];
boundary(1).s      = -s(1); 
boundary(1).min    = mn;
boundary(1).max    = [-s(1);mx(2);mx(3)];
boundary(1).name   = 'x plane 1';
boundary(1).type   = 'xp_plane';

boundary(2).v      = [];
boundary(2).s      = s(1); 
boundary(2).min    = [s(1);mn(2);mn(3)];
boundary(2).max    = mx;
boundary(2).name   = 'x plane 2';
boundary(2).type   = 'xm_plane';

boundary(3).v      = [];
boundary(3).s      = -s(2); 
boundary(3).min    = mn;
boundary(3).max    = [mx(1);-s(2);mx(3)];
boundary(3).name   = 'y plane 1';
boundary(3).type   = 'yp_plane';

boundary(4).v      = [];
boundary(4).s      = s(2); 
boundary(4).min    = [mn(1);s(2);mn(3)];
boundary(4).max    = mx;
boundary(4).name   = 'y plane 2';
boundary(4).type   = 'ym_plane';

boundary(5).v      = [];
boundary(5).s      = -s(3); 
boundary(5).min    = mn;
boundary(5).max    = [mx(1);mx(2);-s(3)];
boundary(5).name   = 'z plane 1';
boundary(5).type   = 'zp_plane';

boundary(6).v      = [];
boundary(6).s      = s(3); 
boundary(6).min    = [mn(1);mn(2);s(3)];
boundary(6).max    = mx;
boundary(6).name   = 'z plane 2';
boundary(6).type   = 'zm_plane';


write_geometry('Box',boundary(find(open)),rg,gname,radius);
