function  make_geom(name,radius,mx,gname)
% make_geom('miyanomori',.019,[-160 -5]);
% Read the curve data from the m file function 'name'
% And create a geometry description file suitable for dem
% radius specifies the curve offset 
% mx specifies how mcuh of the curve to include in the geometry

% later a rotation algorithm should be included to fit the box
% precisely to the selected region of the ski jump
% produce a data file for the simulation cooresponding to
% the miyanomori ski jump,
% Three types of surface are accounted for
% orthogonal planes : planes orthogonal to a coordinate axis
% general plane n.x = lambda
% cylinder elements cylinder axis and radius is specified
% x is along the ski jump
% y across the ski jump
% z is the height of the ski jump
% possible rotation in x,z plane to minimise volume
if nargin < 3
  mx = [-inf inf];
  mx = [-200 10];
end
if nargin < 2
  radius = 0 
end
if nargin < 1
  name = 'miyanomori'
end
if nargin < 4
  gname = [name '.geom'];
end

SRT = 1; % Straight line
ARC = 2; % Circular arc
CBS = 3; % Cubic spline

width = inf;

eval(['curve = ' name ';']);       % Read in the curve data

if strcmp(name,'miyanomori')
  curve(11)=[]; % Remove the take off drop
end

% Offset the curve by the radius
curve = offset_curve(curve,radius);

e = [];
cpp = [];
for i=1:length(curve)
  if (curve(i).q(1)<=mx(1)) | (curve(i).p(1)>=mx(2))
    e = [e i]; % Which ones to eliminate
  else
    cpp = [cpp curve(i).p curve(i).q];
  end
end
nc = length(curve);
curve(e) = []; % Eliminate the unneeded curve elements;

plot_curve(curve);

% Convert the 2d curve to a 3d boundary that stretches forever
b = curve_to_bound(curve,-inf,-width/2,width/2);
nb= length(b);
b(1).min(1)=-inf; % Extend forever
b(nb).max(1)=inf;

range = [mx(1) -8  min(cpp(2,:)) mx(2) 8 max(cpp(2,:))];  
range =[];
% Now write this out to a file in the correct format
% Only make cuts along the x axis
write_geometry(name,b,range,gname,radius,[],[1 0 0]);

function b = curve_to_bound(c,minz,miny,maxy);
SRT = 1; % Straight line
ARC = 2; % Circular arc
CBS = 3; % Cubic spline
% Creates a boundary below the curve to minz between miny and
% maxy
% If these are absent uses +-inf
if nargin < 2
  maxy = inf;
end
if nargin < 3
  miny = -inf;
end
if nargin < 2
  minz = -inf;
end

    
maxparam = 2; % Number of vector and scalar parameters
b(1).v = zeros(3,0);
b(1).s = zeros(1,0);
b(1).name = ' ';
for h=1:length(c) % step through the line elements
  b(h).name = c(h).name;
  tr = [c(h).p(1) miny c(h).p(2);
        c(h).q(1) miny c(h).q(2);
	c(h).p(1) maxy minz];
  b(h).min = min(tr)';
  b(h).max = max(tr)';
  
  switch c(h).type 
  case SRT,  %Straight line resulting in a plane
    % equation is n'*x=p
    n = [c(h).n(1) 0 c(h).n(2)];  % Calculate outward normal
    l = c(h).n'*c(h).p;           % Calculate normal parameter
    if abs(l-c(h).n'*c(h).q )>0.001
      error('problem calculating plane equation')
    end
    b(h).s(1) = l;              % n'*x=l plane equation
    b(h).v(:,1) = [c(h).n(1) 0 c(h).n(2)];% normal
    b(h).type='plane';            % general plane
  case ARC, % A circular arc of specified radius and center
    % equation is (n'*x-l)^2 + (m'*x)^2=r^2
    r = c(h).r; % radius
    if r>0   b(h).type='p_cylinder';
    else     b(h).type='m_cylinder';
    end
    n = [c(h).c(1);0;c(h).c(2)];l=norm(n);n=n/l; % vector to centre
    m = [-c(h).c(2);0;c(h).c(1)]/l;           % orthonormal vector
    b(h).v(:,1)=n;                 % Vector to centre
    b(h).s(1)=l;                   % Distance to centre
    b(h).v(:,2)=m;                 % Vector to centre
    b(h).s(2)=abs(r);              % Radius of cylinder
  end % End case
end
