function [r1_name,r2_name]=make_sim(out_name,simul_num,real_num,sep,base_name)
%function [r1_name,r2_name]=make_sim(out_name,simul_num,real_num,sep,base_name)
% Creates a simulation at Miyanomori with name and simul_num
% balls approximating real_num balls
% Creates two sets of files for the two stages of the simulation
% First the balls are dropped into the box
% Then the balls are allowed to role down the slope
% Sep is the distance down the slope the box is

if nargin < 1
  out_name = 'miya_test';
end

if nargin < 2
  simul_num = 100
end

if nargin < 3
  real_num = simul_num
end

if nargin < 4
  sep = 30
end

if nargin < 5
  base_name = '/home/jim/matlab/dem/base.param'
end

box = [3.6 2.7 1.8]; % Dimensions of the box

sep = -112.059*sin(sep/112.059); % How far down the slope the box is

g1_name   = [out_name 'a.geom'];
g2_name   = [out_name 'b.geom'];
p1_name   = ['/home/jim/dem/data/' out_name 'a.param'];
p2_name   = ['/home/jim/dem/data/' out_name 'b.param'];
r1_name   = [out_name 'a'];
r2_name   = [out_name 'b'];

x = ping_pong({'radius' 'e_gravity' 'mass'}); % Get the particle properties
radius = x(1);
gravity = x(2);
mass = x(3);

mass = real_num/simul_num * mass;
radius = (real_num/simul_num)^(1/3) * radius;

vol = (simul_num/prod(box))^(1/3);
pd = round(box*vol); % Particle distribution
while(prod(pd)<=simul_num) % If the box isn't big enough
  [diff,ind] = max(box*vol-pd);
  pd(ind) = pd(ind)+1;
end
if any(2*radius*pd>box) % If any of the balls don't fit in the box
  pd(1:2) = floor(box(1:2)/(2*radius));
  pd(3) = ceil(simul_num/prod(pd(1:2)));
end


if simul_num*(4*radius^3) > prod(box)
  error('The balls can''t fit in the box')
end

% M file to calculate the box position and angle
% specifies 

hd = box/2;

x1 = sep-box(1);  % X positions for box
x2 = sep;  % X positions for box



[curve,width] = miyanomori;    % get curve information
[z dz] = plot_curve(curve,x1); % Calculate box position
z1 = z;
theta = atan(dz); % Angle of box
ct = cos(theta);
st = sin(theta);
xs = [x1   x1+box(1)*ct   x1+box(1)*ct-box(3)*st   x1-box(3)*st];
zs = [z1   z1+box(1)*st   z1+box(1)*st+box(3)*ct   z1+box(3)*ct];
mx = mean(xs);
mz = mean(zs);
clf
plot(xs,zs)
hold on
x = sep+(-50:0.1:50);
[z dz] = plot_curve(curve,x); %
plot(x,z)
figure
% Make the geometry files
make_cuboid_geom(box,radius,[1 1 1;1 1 0],inf,g1_name);
make_geom('miyanomori',radius,[-160 max(xs)+5],g2_name);
delete(gcf)

% Now read in the input data files and make the output
fid = fopen(base_name,'r');
program_param = read_all_name_list(fid,'PROGRAM_PARAM');
record_param = read_all_name_list(fid,'RECORD_PARAM');
particle_param = read_all_name_list(fid,'PARTICLE_PARAM');
pp_param = read_all_name_list(fid,'COLLISION_PARAM');
pw_param = read_all_name_list(fid,'COLLISION_PARAM');
air_param = read_all_name_list(fid,'AIR_PARAM');
termination_param = read_all_name_list(fid,'TERMINATION_PARAM');
particle_state = read_all_name_list(fid,'PARTICLE_STATE');
fclose(fid);


disp(['Writing parameter file ' p1_name]);
fid = fopen(p1_name,'w');

% Choose special contact numbers to hasten equilibrium
pp1 = pp_param;
pw1 = pw_param;
pp1.friction = 0;
pp1.restitution = 0.1;
pw1.friction = 0;
pw1.restitution  = 0.1;
program_param.geom_file_name = g1_name;
record1.rec_name = r1_name; % Record nothing
particle_param.p_radius = radius;
particle_param.p_mass = mass;
particle_param.p_gravity = -gravity * [st 0.01 ct];
term1.max_t = 5;
init1.origin = [0 0 pd(3)*radius-box(3)/2];
init1.initial_type = 'cuboid';
init1.cube_size    = pd;
init1.particle_num = simul_num;
init1.random_lin_vel = 'T';
init1.lin_vel_max = 5;



write_name_list(fid,program_param,'PROGRAM_PARAM');
write_name_list(fid,record1,'RECORD_PARAM');
write_name_list(fid,particle_param,'PARTICLE_PARAM');
write_name_list(fid,pp1,'COLLISION_PARAM');
write_name_list(fid,pw1,'COLLISION_PARAM');
write_name_list(fid,air_param,'AIR_PARAM');
write_name_list(fid,term1,'TERMINATION_PARAM');
write_name_list(fid,init1,'PARTICLE_STATE');
fclose(fid);


program_param.geom_file_name = g2_name;
record_param.rec_name = r2_name;
particle_param.p_gravity = [0 0 -gravity];
init2.initial_type    = 'data';
init2.file_name       = [r1_name '.dump'];
init2.origin          = [mx 0 mz];
init2.rotation = [0 -theta 0];
init2.particle_num = simul_num;

disp(['Writing parameter file ' p2_name]);
fid = fopen(p2_name,'w');
write_name_list(fid,program_param,'PROGRAM_PARAM');
write_name_list(fid,record_param,'RECORD_PARAM');
write_name_list(fid,particle_param,'PARTICLE_PARAM');
write_name_list(fid,pp_param,'COLLISION_PARAM');
write_name_list(fid,pw_param,'COLLISION_PARAM');
write_name_list(fid,air_param,'AIR_PARAM');
write_name_list(fid,termination_param,'TERMINATION_PARAM');
write_name_list(fid,init2,'PARTICLE_STATE');
fclose(fid);
