function  [x,z,dz] = plot_curve(c,x)
% Plot the data in the curve c over it's range
% or at points specified by x

SRT = 1; % Straight line
ARC = 2; % Circular arc
CBS = 3; % Cubic spline

for i=1:length(c)
  cxp(i) = c(i).p(1);
  cxq(i) = c(i).q(1);
end

if nargin<2
  npoints = 1000;
  minx = min([cxp cxq]);
  maxx = max([cxp cxq]);
  x = minx:(maxx-minx)/(npoints-1):maxx;
end
y  = repmat(NaN,[length(c) length(x)]);
dy = repmat(NaN,[length(c) length(x)]);
z  = zeros(1,length(x));
dz = zeros(1,length(x));

for h=1:length(c)
  p = c(h).p;
  q = c(h).q;
  dp = c(h).dp;
  f = find((x>=c(h).p(1)) & (x<= c(h).q(1)));
  switch c(h).type
  case SRT, % Straight line
    if dp(1) ~= 0
      g = dp(2)/dp(1);
    else
      g = 0;
    end
    y(h,f) = p(2)- g*p(1)+g*x(f);
    dy(h,f) = repmat(g,[1 length(f)]);
  case ARC,             % A circular arc of specified radius
    r = c(h).r;
    a = c(h).c;
    y(h,f)  = a(2) - r*sqrt(1 - (x(f)-a(1)).^2/r^2);
    dy(h,f) = (x(f)-a(1))./(r*sqrt(1 - (x(f)-a(1)).^2/r^2));
  end
  z(f) = y(h,f);
  dz(f) = dy(h,f);
end
if nargout==0
  subplot(2,1,1)
  hold on
  plot(x,y)
  hold on
  subplot(2,1,2)
  plot(x,dy)
  z = [];dz=[];
end

