module col_props_f90_def
  use precision
  use program_units
  implicit none
  !private SR2201BUG

  ! This type hold all the parameters necesary for calculating and checking 
  ! a collision
  type, public :: col_props_t
     real(r_kind)  :: myu         ! Coefficient of friction
     real(r_kind)  :: myu_vel     ! friction velocity
     real(r_kind)  :: nk          ! Normal spring constant
     real(r_kind)  :: neta        ! Normal damping constant
  end type col_props_t
  interface conv_to_si
     module procedure conv_col_props_to_si_s, conv_col_props_to_si_a, &
          conv_col_props_to_si_aa
  end interface
  interface conv_from_si
     module procedure conv_col_props_from_si_s, conv_col_props_from_si_a, &
          conv_col_props_from_si_aa
  end interface
contains
  subroutine conv_col_props_to_si_s(cp)
    type(col_props_t), intent(inout) :: cp
    call conv_to_si(cp%neta,T=-1,M=1)
    call conv_to_si(cp%nk,T=-2,M=1)  
    call conv_velocity_to_si(cp%myu_vel)
  end subroutine conv_col_props_to_si_s

  subroutine conv_col_props_to_si_a(cp)
    type(col_props_t), intent(inout) :: cp(:)
    call conv_to_si(cp%neta,T=-1,M=1)
    call conv_to_si(cp%nk,T=-2,M=1)  
    call conv_velocity_to_si(cp%myu_vel)
  end subroutine conv_col_props_to_si_a

  subroutine conv_col_props_to_si_aa(cp)
    type(col_props_t), intent(inout) :: cp(:,:)
    call conv_to_si(cp%neta,T=-1,M=1)
    call conv_to_si(cp%nk,T=-2,M=1)  
    call conv_to_si(cp%myu_vel,T=-1,L=1)
  end subroutine conv_col_props_to_si_aa

  subroutine conv_col_props_from_si_s(cp)
    type(col_props_t), intent(inout) :: cp
    call conv_from_si(cp%neta,T=-1,M=1)
    call conv_from_si(cp%nk,T=-2,M=1)  
    call conv_velocity_from_si(cp%myu_vel)
  end subroutine conv_col_props_from_si_s

  subroutine conv_col_props_from_si_a(cp)
    type(col_props_t), intent(inout) :: cp(:)
    call conv_from_si(cp%neta,T=-1,M=1)
    call conv_from_si(cp%nk,T=-2,M=1)  
    call conv_velocity_from_si(cp%myu_vel)
  end subroutine conv_col_props_from_si_a

  subroutine conv_col_props_from_si_aa(cp)
    type(col_props_t), intent(inout) :: cp(:,:)
    call conv_from_si(cp%neta,T=-1,M=1)
    call conv_from_si(cp%nk,T=-2,M=1)  
    call conv_from_si(cp%myu_vel,T=-1,L=1)
  end subroutine conv_col_props_from_si_aa



end module col_props_f90_def
 
