#include "dimension.h"
! Contains all the functions do to with MPI
! comms_initialize() 
!     starts MPI and reads number of processes and sets a master process
!     sets the global variables comms_rank
!                               comms_name
!                               comms_size
!                               comms_shape
!                               comms_master
! comms_broadcast(v) 
!     transmits the variable v from the master process to all other processes
!     overloaded so as to cope with all type that we may want to transmit
! comms_max(v) 
!     returns the max onto the master processor 
! comms_min(v) 
!     returns the min onto the master processor
! comms_collect(v,d) 
!     collects a copy of each v into the array d on the master processor
! comms_exchange_part(rop,rup,sop,sup)
!     exchange owning and using particles between processors
! comms_end_exchange_part()
!     wait until particle exchange is over

module comms
  use precision
  use vector
  use sysdep_io
  use particle
  use program_units
  use locate
  use mpi
  implicit none

!  public :: comms_rank, comms_size, comms_shape 
!  public :: comms_master, comms_name, comms_root 
!  
!  public :: comms_initialize, 
!  public :: comms_broadcast, comms_collect
!  public :: comms_exchange_part, comms_end_exchange_part
!  public :: comms_max, comms_min
!  interface comms_broadcast
!     module procedure r_broadcast, i_broadcast, l_broadcast, &
!          ar_broadcast, ai_broadcast, al_broadcast, &
!          rvec_broadcast, ivec_broadcast, locate_t      _broadcast
!  end interface
          
  ! mpi data types that we define

  type(ivec_t), parameter :: shapes(12) = (/&       ! Pick a shape
     ivec_t(ARG(1,1,1)), ivec_t(ARG(1,2,1)), ivec_t(ARG(1,3,1)), ivec_t(ARG(2,2,1)), &
     ivec_t(ARG(1,5,1)), ivec_t(ARG(2,3,1)), ivec_t(ARG(1,7,1)), ivec_t(ARG(2,2,2)), &
     ivec_t(ARG(3,3,1)), ivec_t(ARG(2,5,1)), ivec_t(ARG(1,11,1)), ivec_t(ARG(3,2,2))/)


  integer   :: comms_rank                             ! Processor rank
  character(len=FMPI_MAX_PROCESSOR_NAME) :: comms_name ! Processor name
  integer   :: comms_size                             ! No. of processors
  type(ivec_t) :: comms_shape                          ! Processor shape
  logical   :: comms_master                           ! Am I the master 
  integer, parameter :: comms_root = fmpi_root_rank   ! Which processor is root

contains
  subroutine comms_initialize
    integer n
    call fmpi_init
    call fmpi_comm_rank(comms_rank)
    call fmpi_comm_size(comms_size)
    call fmpi_get_processor_name(comms_name,n)
    if (fmpi_is_root(comms_rank)) then
       comms_master = .true.
    else
       comms_master = .false.
    end if
    comms_shape=shapes(comms_size) ! Set the geometry shape
   end subroutine comms_initialize
  subroutine comms_finalize
    call fmpi_finalize
  end subroutine comms_finalize
  function displ_sum(a) result(b)
    integer,intent(in) :: a(:)
    integer :: b(size(a))
    integer i
    b(1)=0
    do i=1,size(a)-1
       b(i+1) = b(i) + a(i)
    end do
  end function displ_sum
end module comms
