#include "part_props.h"
module min_part_f90_def
  use vector
  use col_props_f90_def
  use program_units
  implicit none 
  type, public :: min_part_t
     integer pn   ! The global number identifying this particle
     type(rvec_t) r  ! position 
     type(rvec_t) v  ! linear velocity 
     type(rang_t) w  ! angular velocity 
#ifdef PARTICLE_HAS_TYPE
     integer type    ! What type of particle it is
#endif
#ifdef PARTICLE_HAS_PHYS_PROPS
     real(r_kind) radius ! Particle radius
     real(r_kind) imass  ! 1/particle_mass
     real(r_kind) imi    ! particle_radius^2/particle_moment_of_inertia
#endif
#ifdef PARTICLE_HAS_COL_PROPS
     type(col_props_t) cp ! Collision properties
#endif
  end type min_part_t

  public :: conv_to_si
  public :: conv_from_si
  public :: write_part_props
  public :: read_part_props
  public :: read_part_state
  public :: write_part_state


  interface conv_to_si
     module procedure    ampt_conv_to_si
  end interface
  interface conv_from_si
     module procedure  ampt_conv_from_si
  end interface

contains
  subroutine read_part_state(fp,p)
    integer, intent(in) :: fp
    type(min_part_t), intent(out) :: p(:)
    integer i
    p%r = null_rvec
    p%v = null_rvec
    p%w = null_rang
    do i = 1,size(p)
       read(fp,*)p(i)%pn,p(i)%r,p(i)%v,p(i)%w
    end do
    p%w = p%w*P_RADIUS(p)
  end subroutine read_part_state
  subroutine write_part_state(fp,p)
    integer, intent(in) :: fp
    type(min_part_t), intent(in) :: p(:)
    integer i
    do i = 1,size(p)
       write(fp,*)p(i)%pn,p(i)%r,p(i)%v,p(i)%w/P_RADIUS(p(i))
   end do
   end subroutine write_part_state
  subroutine write_part_props(fp,p)
    integer, intent(in) :: fp
    type(min_part_t), intent(in) :: p(:)
    integer i
    do i = 1,size(p)
       write(fp,*)p(i)%pn, P_TYPE(p(i)), P_RADIUS(p(i)),P_MASS(p(i)),&
            P_MI(p(i))*P_RADIUS(p(i))**2
    end do
  end subroutine write_part_props
  subroutine read_part_props(fp,p)
    integer, intent(in) :: fp
    type(min_part_t), intent(out) :: p(:)
    integer i,j
    do i = 1,size(p)
#ifdef PARTICLE_HAS_PHYS_PROPS
#ifdef PARTICLE_HAS_TYPE
       read(fp,*)p(i)%pn, p(i)%type, p(i)%radius, p(i)%imi, p(i)%imass
#else
       read(fp,*)p(i)%pn, j, p(i)%radius, p(i)%imi, p(i)%imass
#endif
#else
#ifdef PARTICLE_HAS_TYPE
       read(fp,*)p(i)%pn, p(i)%type
#else
       read(fp,*)p(i)%pn
#endif
#endif
    end do
#ifdef PARTICLE_HAS_PHYS_PROPS
    p%imi = p%radius**2/p%imi
    p%imass = 1/p%imass
#endif
  end subroutine read_part_props
  subroutine ampt_conv_to_si(p)
    type(min_part_t), intent(inout) :: p(:)
    call conv_length_to_si(p%r)
    call conv_velocity_to_si(p%v)
    call conv_velocity_to_si(p%w)
#ifdef PARTICLE_HAS_PHYS_PROPS
    call conv_length_to_si(p%radius)
    call conv_to_si(p%imass,M=-1)
    call conv_to_si(p%imi,M=-1)
#endif
#ifdef PARTICLE_HAS_COL_PROPS
    call conv_to_si(p%cp) ! Collision properties
#endif
  end subroutine ampt_conv_to_si
 subroutine ampt_conv_from_si(p)
    type(min_part_t), intent(inout) :: p(:)
    call conv_length_from_si(p%r)
    call conv_velocity_from_si(p%v)
    call conv_velocity_from_si(p%w)
#ifdef PARTICLE_HAS_PHYS_PROPS
    call conv_length_from_si(p%radius)
    call conv_from_si(p%imass,M=-1)
    call conv_from_si(p%imi,M=-1)
#endif
#ifdef PARTICLE_HAS_COL_PROPS
    call conv_from_si(p%cp) ! Collision properties
#endif
  end subroutine ampt_conv_from_si
end module min_part_f90_def

