#include "units.h" 
#include "radius.h"
! There are several possibilities
! All the particles are identical
!     All the properties are stored out of the particle derived typs
! There are a small number of different types of particles 
!        mass, radius, moment of inertia can be stored in the particle type or
!        in a seperate list  
!        collision properties are stored in a two dimensional array
!  There are a large number of different types of particles
!       All properties are stored in the particle type
!       collision properties are dynamically calculated
!
! Flags are PARTICLE_HAS_COL_PROPS
! Flags are PARTICLE_HAS_PHYS_PROPS

! Default is ONE_TYPE
#ifndef ONE_TYPE
#ifndef SMALL_TYPE
#ifndef LARGE_TYPE
#define ONE_TYPE
#endif
#endif
#endif

#ifndef SOLID
#ifndef SHELL
#define SHELL
#endif
#endif 


#ifdef ONE_TYPE
#define GET_P_COLL_PROP(p) p_col_props(1)
#define GET_PP_COLL_PROP(p,q) pp_col_props(1,1)
#define GET_PW_COLL_PROP(p,w) pw_col_props(1,w)
#else
#ifdef SMALL_TYPE
#define PARTICLE_HAS_TYPE
#define PARTICLE_HAS_PHYS_PROPS
#define GET_P_COLL_PROP(p)    p_col_props(p%type)
#define GET_PP_COLL_PROP(p,q)    pp_col_props(p%type,q%type)
#define GET_PW_COLL_PROP(p,w)    pw_col_props(p%type,w)
#else
#ifdef  LARGE_TYPE
#define PARTICLE_HAS_PHYS_PROPS
#define PARTICLE_HAS_COL_PROPS
#define GET_P_COLL_PROP(p)    p_col_props(p%type)
#define GET_PP_COLL_PROP(p,q) add_pp_col_props(p,q)
#define GET_PW_COLL_PROP(p)   add_pw_col_props(p,wn)
#endif
#endif
#endif

! For shells scaled moment of inertia is just mass
! For solid  particles 3/5M 
#ifdef SHELL
#define MAX_MI      MAX_MASS
#else
#define MAX_MI      0.6*MAX_MASS
#endif

#ifdef PARTICLE_HAS_PHYS_PROPS
#define P_RADIUS(p)  p%radius
#define P_IMASS(p)   p%imass
#define P_MASS(p)    1/p%imass
#define P_IMI(p)     p%imi
#define P_MI(p)      1/p%imi
#else
#define P_RADIUS(p)  MAX_RADIUS
#define P_IMASS(p)   1/MAX_MASS
#define P_MASS(p)    MAX_MASS
#define P_IMI(p)     MAX_MI
#define P_MI(p)      1/MAX_MI
#endif

#ifdef PARTICLE_HAS_TYPE
#define P_TYPE(p)    p%type
#else
#define P_TYPE(p)    1
#endif 
