#include "dimension.h"
#include "part_props.h"
! What is the kissing number? Maximum number of particle contacts.
#ifdef ONE_TYPE
#ifdef TWODIM
#define MAX_CON  6 
#else
#ifdef THREEDIM
#define MAX_CON  12
#endif
#endif
#endif

#ifdef SMALL_TYPE
#define RANGLE=asin(1/(RADIUS_RATIO+1)) 
#ifdef TWODIM
#define MAX_CON  floor(PI/RANGLE) 
#else
#ifdef THREEDIM
#define MAX_CON floor(12*(1-sqrt(3)/2)/(1-cos(RANGLE)) 
#endif
#endif
#undef MAX_CON
#define MAX_CON 24
#endif

! This module contains the routines for the particle dynamics
! This module defines the type for holding all the data about particles
! and a type containing the minimum information about particles.
! The minimum type is designed for message passing and input and output 
! files 
! It also defines a list type for both of these types and functions for 
! creating these lists and popping and pushing elements and arrays as well
! as overloading operators.
! equals can translate between the two particle types and equate lists
! add can be used for adding lists of the same type together
! The lists are defined with a set size and this can be automatically 
! increased with a warning if the list is overextended
! functions overlo
! There are two main types of list corresponding to the 
 ! minimum particle type or the full particle type. Most functions are overloaded
! push 
! how to access is difficult to decide

module particle
  use precision
  use vector
  use sysdep_io
  use min_part_f90_def
  use program_units

  implicit none
  !private SR2201BUG
  logical, private, parameter :: AUTO_ALLOCATE = .false.

  ! Public functions
  public :: push, pop, check_bounds, alloc , dealloc, resize, increase, print_particle

  public :: assignment(=)

  ! in function names
  ! pt   : particle type
  ! mpt  : minimum particle type
  ! plt  : particle list
  ! mplt : minimum particle list
  ! pat  : array of particle type
  ! mpat : array of minimum particle type

  interface check_bounds
     module procedure pat_check_bounds, mpat_check_bounds
  end interface

  interface alloc
     module procedure alloc_plt, alloc_mplt
  end interface
  interface dealloc
     module procedure dealloc_plt, dealloc_mplt
  end interface
  interface resize
     module procedure resize_plt, resize_mplt
  end interface
  interface increase
     module procedure increase_plt, increase_mplt
  end interface
  ! Inititialise lists with scalars, arrays or other lists
  interface assignment(=)
     module procedure  &
          mpt_equ_pt,   pt_equ_mpt,  mpat_equ_pat, pat_equ_mpat, &
          & plt_equ_plt, mplt_equ_plt, plt_equ_mplt, mplt_equ_mplt, &
          & plt_equ_pat, mplt_equ_pat, plt_equ_mpat, mplt_equ_mpat, &
          & pat_equ_plt, mpat_equ_plt, pat_equ_mplt, mpat_equ_mplt
  end interface

  ! Add scalars, arrays or other lists onto lists
  interface push
     module procedure &
          & plt_push_pt,   plt_push_pat,   plt_push_plt, &
          & plt_push_mpt,  plt_push_mpat,  plt_push_mplt, &
          & mplt_push_pt,  mplt_push_pat,  mplt_push_plt, &
          & mplt_push_mpt, mplt_push_mpat, mplt_push_mplt
  end interface

 
  interface conv_to_si
     module procedure  energy_t_conv_to_si
  end interface
  interface conv_from_si
     module  procedure   energy_t_conv_from_si
  end interface
 

  ! Remove a single element or an array from a list
  ! The array must be strictly increasing
  interface remove
     module procedure splt_remove, aplt_remove, &
          & smplt_remove, amplt_remove
  end interface

  ! Remove and return a single element or an array from a list
  ! The array must be strictly increasing
  interface pop
     module procedure splt_pop, aplt_pop, &
          & smplt_pop, amplt_pop
  end interface
  interface isfinite
     module procedure isfinite_p, isfinite_pa
  end interface 
  !SR2201BUG
  ! return how many particles in a list
  !   interface size
  !      module procedure plt_size, mplt_size
  !   end interface

  ! particle structure definition 
  type, public :: particle_t 
     type(min_part_t) mp ! number, position, velocity etc...
     integer grid   ! which grid cell it is currently in 
     integer nc     ! Number of contacts 
     integer srn(MAX_CON) ! Which particles it's in contact with 
     type(rvec_t) f  ! resultant force 
     type(rang_t) am ! resultant moment 
     real(r_kind) cpe ! potential energy of the particles contacts 
  end type particle_t

  type, public :: part_list_t
     type(particle_t), pointer :: p(:) ! array pointing to in use elements
     type(particle_t), pointer :: all(:) !larger allocated block
  end type part_list_t

  type, public :: min_part_list_t
     type(min_part_t), pointer :: p(:) ! array pointing to in use elements
     type(min_part_t), pointer :: all(:) !larger allocated block
  end type min_part_list_t

  type, public :: energy_t
     real(r_kind) lke    ! Linear kinetic energy 
     real(r_kind) ake    ! Angular Kinetic energy 
     real(r_kind) gpe    ! Gravitational potential energy 
     real(r_kind) cpe    ! Contacts  potential energy 
     real(r_kind) tke    ! Total kinetic energy 
     real(r_kind) tpe    ! Total potential energy 
     real(r_kind) total  ! Total energy 
  end type energy_t
  ! Special program types

  logical, parameter :: debug_p_lists=.false.
contains      
  function plt_size(p) result(r)
    type(part_list_t) p
    integer r
    r = size(p%p)
  end function plt_size
  function mplt_size(p) result(r)
    type(min_part_list_t) p
    integer r
    r = size(p%p)
  end function mplt_size
  subroutine print_particle(fp,p)
    integer, intent(in)          :: fp
    type(particle_t), intent(in) :: p
    write(fp,*)'particle no.', p%mp%pn, 'grid sq', p%grid
    if (p%nc == 0) then
       write(fp,*)'no contacts'
    else
       write(fp,*)'In contact with',p%srn(1:p%nc)
    end if
    write(fp,*)'position', p%mp%r
    write(fp,*)'l velocity', p%mp%v
    write(fp,*)'a velcoity ', p%mp%w
    write(fp,*)'force', p%f
    write(fp,*)'moment', p%am
    write(fp,*)'p.e.', p%cpe
  end subroutine print_particle
  ! Allocs a particle list with a elements and space for up to b elements
  subroutine alloc_plt(c, a, bb)
    type(part_list_t), intent(out) :: c
    integer,intent(in) :: a
    integer,intent(in), optional :: bb
    integer b
    if (present(bb)) then 
       b = bb
    else
       b = a
    end if
    allocate(c%all(b))        ! Allocate the space
    if (debug_p_lists) write(std_out,*)'Allocating particle list size',a,',up to',b 
    c%p => c%all(1:a)  ! Point at the actual number of elements
  end subroutine alloc_plt
  subroutine alloc_mplt(c,a,bb)
    type(min_part_list_t),intent(out) :: c
    integer,intent(in) :: a
    integer,intent(in), optional :: bb
    integer b
    if (present(bb)) then 
       b = bb
    else
       b = a
    end if
    allocate(c%all(b))        ! Allocate the space
    if (debug_p_lists) write(std_out,*)'Allocating min particle list size',a,',up to',b 
    c%p => c%all(1:a)  ! Point at the actual number of elements
  end  subroutine alloc_mplt
  subroutine dealloc_plt(a)
    type(part_list_t), intent(inout) :: a
    if (debug_p_lists) write(std_out,*)'Deallocating particle list size',size(a%p),&
         ',up to',size(a%all)
    nullify(a%p)
    deallocate(a%all)
  end subroutine dealloc_plt
  subroutine dealloc_mplt(a)
    type(min_part_list_t), intent(inout) :: a
    if (debug_p_lists) write(std_out,*)'Deallocating min particle list size',size(a%p),&
         ',up to',size(a%all)
    nullify(a%p)
    deallocate(a%all)
  end subroutine dealloc_mplt
  subroutine resize_plt(a, b) ! change the size  of list a pointer to b
    type(part_list_t), intent(inout) :: a
    integer,intent(in) :: b
    call check_bounds(a,b) ! Can we change the size of a to b
    a%p => a%all(1:b)     ! repoint
  end subroutine resize_plt
  subroutine resize_mplt(a, b) ! change the size  of list a pointer to b
    type(min_part_list_t), intent(inout) :: a
    integer,intent(in) :: b
    call check_bounds(a,b) ! Can we change the size of a to b
    a%p => a%all(1:b)     ! repoint
  end subroutine resize_mplt

  ! Allocate extra storage for particle lists
  subroutine increase_plt(a, b) ! change the size  of list a pointer to b
    type(part_list_t), intent(inout) :: a
    integer,intent(in) :: b
    type(part_list_t) :: c
    call info_msg('Resizing particle list')
    call alloc(c,size(a%p),b) ! Create a new particle list
    c%p = a%p                ! Copy the data across
    deallocate(a%all)        ! Deallocate the old structure
    a%all => c%all           ! Change the pointer
    a%p => c%p               ! Change the pointer
  end subroutine increase_plt
  subroutine increase_mplt(a, b) ! Increase the size of a
    type(min_part_list_t), intent(inout) :: a
    integer,intent(in) :: b
    type(min_part_list_t) :: c
    call info_msg('Resizing particle list')
    call alloc(c,size(a%p),b) ! Create a new particle list
    c%p = a%p                ! Copy the data across
    deallocate(a%all)        ! Deallocate the old structure
    a%all => c%all           ! Change the pointer
    a%p => c%p               ! Change the pointer
  end subroutine increase_mplt

  ! Check there is space for b elements in array a
  subroutine pat_check_bounds(a,b)
    type(part_list_t), intent(inout) :: a
    integer, intent(in) :: b
    if (AUTO_ALLOCATE) then
       if (size(a%all)<b) call increase(a,b)
    else
       if (size(a%all)<b) then
          call error_halt('Too large assignment in particle.f90')
       end if
    end if ! Auto allocate enough space for the list
  end subroutine pat_check_bounds
  subroutine mpat_check_bounds(a,b)
    type(min_part_list_t), intent(inout) :: a
    integer, intent(in) :: b
    if (AUTO_ALLOCATE) then
       if (size(a%all)<b) call increase(a,b)
    else
       if (size(a%all)<b) then
          call error_halt('Too large assignment in particle.f90')
       end if
    end if ! Auto allocate enough space for the list
  end subroutine mpat_check_bounds

  ! Conversion and copying between particle_t and min_part_t
  subroutine mpt_equ_pt(a, b)
    type(min_part_t), intent(out) :: a
    type(particle_t), intent(in)  :: b
    a = b%mp
  end subroutine mpt_equ_pt
  subroutine pt_equ_mpt(a, b)
    type(particle_t), intent(out) :: a
    type(min_part_t), intent(in)  :: b
    a%mp = b
  end subroutine pt_equ_mpt
  subroutine mpat_equ_pat(a, b)
    type(particle_t), intent(in) :: b(:)
    type(min_part_t), intent(out):: a(size(b))
    a = b%mp
  end subroutine mpat_equ_pat
  subroutine pat_equ_mpat(a, b)
    type(min_part_t), intent(in)  :: b(:)
    type(particle_t), intent(out) :: a(size(b)) 
    a%mp = b
  end subroutine pat_equ_mpat

  ! Conversion and copying between lists of particle_t and min_part_t
  subroutine plt_equ_plt(a, b)
    type(part_list_t), intent(inout) :: a
    type(part_list_t), intent(in)  :: b
    call resize(a, size(b%p))
    a%p = b%p;
  end subroutine plt_equ_plt
  subroutine mplt_equ_plt(a, b)
    type(min_part_list_t), intent(inout) :: a
    type(part_list_t), intent(in)  :: b
    call resize(a, size(b%p))
    a%p = b%p;
  end subroutine mplt_equ_plt
  subroutine plt_equ_mplt(a, b)
    type(part_list_t), intent(inout) :: a
    type(min_part_list_t), intent(in)  :: b
    call resize(a, size(b%p))
    a%p = b%p;
  end subroutine plt_equ_mplt
  subroutine mplt_equ_mplt(a, b)
    type(min_part_list_t), intent(inout) :: a
    type(min_part_list_t), intent(in)  :: b
    call resize(a, size(b%p))
    a%p = b%p;
  end subroutine mplt_equ_mplt

  ! Conversion and copying between from an array to a list
  subroutine plt_equ_pat(a, b)
    type(part_list_t), intent(inout) :: a
    type(particle_t), intent(in)   :: b(:)
    call resize(a, size(b))
    a%p = b;
  end subroutine plt_equ_pat
  subroutine mplt_equ_pat(a, b)
    type(min_part_list_t), intent(inout) :: a
    type(particle_t), intent(in)       :: b(:)
    call resize(a, size(b))
    a%p = b;
  end subroutine mplt_equ_pat
  subroutine plt_equ_mpat(a, b)
    type(part_list_t), intent(inout)   :: a
    type(min_part_t), intent(in) :: b(:)
    call resize(a, size(b))
    a%p = b;
  end subroutine plt_equ_mpat
  subroutine mplt_equ_mpat(a, b)
    type(min_part_list_t), intent(inout) :: a
    type(min_part_t), intent(in)   :: b(:)
    call resize(a, size(b))
    a%p = b;
  end subroutine mplt_equ_mpat

  ! Conversion and copying from a list to an array
  ! The compiler will choke if the array is the wrong size)
  subroutine pat_equ_plt(a, b)
    type(part_list_t), intent(in) :: b
    type(particle_t), intent(inout) :: a(size(b%p))
    a = b%p
  end subroutine pat_equ_plt
  subroutine mpat_equ_plt(a, b)
    type(part_list_t), intent(in)  :: b
    type(min_part_t), intent(inout) :: a(size(b%p))
    a = b%p
  end subroutine mpat_equ_plt
  subroutine pat_equ_mplt(a, b)
    type(min_part_list_t), intent(in)  :: b
    type(particle_t), intent(out) :: a(size(b%p))
    a = b%p
  end subroutine pat_equ_mplt
  subroutine mpat_equ_mplt(a, b)
    type(min_part_list_t), intent(in)  :: b
    type(min_part_t), intent(inout) :: a(size(b%p))
    a = b%p
  end subroutine mpat_equ_mplt

  ! Functions for pushing onto particle lists
  subroutine plt_push_pt(a,b)
    type(part_list_t), intent(inout) :: a
    type(particle_t), intent(in)     :: b
    call resize(a, size(a%p)+1) ! Check we can add on one more particle
    a%p(size(a%p))=b
  end subroutine plt_push_pt
  subroutine plt_push_pat(a,b)
    type(part_list_t), intent(inout) :: a
    type(particle_t), intent(in)     :: b(:)
    call resize(a,size(a%p)+size(b))    ! Check bounds
    a%p((size(a%p)+1-size(b)):)=b
  end subroutine plt_push_pat
  subroutine plt_push_plt(a,b)
    type(part_list_t), intent(inout) :: a
    type(part_list_t), intent(in)    :: b
    call resize(a,size(a%p)+size(b%p))    ! Check bounds
    a%p((size(a%p)+1-size(b%p)):)=b%p
  end subroutine plt_push_plt
  subroutine plt_push_mpt(a,b)
    type(part_list_t), intent(inout) :: a
    type(min_part_t), intent(in)     :: b
    call resize(a, size(a%p)+1)! Check we can add on one more particle
    a%p(size(a%p))=b
  end subroutine plt_push_mpt
  subroutine plt_push_mpat(a,b)
    type(part_list_t), intent(inout) :: a
    type(min_part_t), intent(in) :: b(:)
    call resize(a,size(a%p)+size(b))
    a%p((size(a%p)+1-size(b)):size(a%p))=b
  end subroutine plt_push_mpat
  subroutine plt_push_mplt(a,b)
    type(part_list_t), intent(inout) :: a
    type(min_part_list_t), intent(in):: b
    call resize(a,size(a%p)+size(b%p))    ! Check bounds
    a%p((size(a%p)+1-size(b%p)):size(a%p))=b%p
  end subroutine plt_push_mplt

  ! Functions for pushing onto min_particle lists
  subroutine mplt_push_pt(a,b)
    type(min_part_list_t), intent(inout) :: a
    type(particle_t), intent(in)         :: b
    call resize(a, size(a%p)+1) ! Check we can add on one more particle
    a%p(size(a%p))=b
  end subroutine mplt_push_pt
  subroutine mplt_push_pat(a,b)
    type(min_part_list_t), intent(inout) :: a
    type(particle_t), intent(in)         :: b(:)
    call resize(a,size(a%p)+size(b))    ! Check bounds
    a%p((size(a%p)+1-size(b)):size(a%p))=b;
  end subroutine mplt_push_pat
  subroutine mplt_push_plt(a,b)
    type(min_part_list_t), intent(inout) :: a
    type(part_list_t), intent(in)        :: b
    call resize(a,size(a%p)+size(b%p))    ! Check bounds
    a%p((size(a%p)+1-size(b%p)):size(a%p))=b%p
  end subroutine mplt_push_plt
  subroutine mplt_push_mpt(a,b)
    type(min_part_list_t), intent(inout) :: a
    type(min_part_t), intent(in)     :: b
    call resize(a,size(a%p)+1)    ! Check bounds
    a%p(size(a%p))=b
  end subroutine mplt_push_mpt
  subroutine mplt_push_mpat(a,b)
    type(min_part_list_t), intent(inout) :: a
    type(min_part_t), intent(in)     :: b(:)
    call resize(a,size(a%p)+size(b))    ! Check bounds
    a%p((size(a%p)+1-size(b)):)=b
  end subroutine mplt_push_mpat
  subroutine mplt_push_mplt(a,b)
    type(min_part_list_t), intent(inout) :: a
    type(min_part_list_t), intent(in)    :: b
    call resize(a,size(a%p)+size(b%p))    ! Check bounds
    a%p((size(a%p)+1-size(b%p)):)=b%p
  end subroutine mplt_push_mplt


  ! Remove single elements from lists and keep the lists contiguous
  ! Removing long lists of unordered elements is difficult
  subroutine splt_remove(a,b)
    type(part_list_t), intent(inout) :: a
    integer :: b
    call assert(b<=size(a%p),&
         'Tried to removed a nonexistant particle from a list')
    a%p(b) = a%p(size(a%p)); ! Copy the last particle to the new position
    call resize(a,size(a%p)-1)
  end subroutine splt_remove
  subroutine smplt_remove(a,b)
    type(min_part_list_t), intent(inout) :: a
    integer :: b
    call assert(b<=size(a%p), &
         'Tried to removed a nonexistant particle from a list')
    a%p(b) = a%p(size(a%p)); ! Copy the last particle to the new position
    call resize(a,size(a%p)-1)
  end subroutine smplt_remove
  subroutine aplt_remove(a,b)
    type(part_list_t), intent(inout) :: a
    integer :: b(:)
    integer i
    call assert(all(b(1:(size(b)-1))<b(2:)),'List is not ordered')
    do i = size(b),1,-1
       a%p(b(i))=a%p(size(a%p)-size(b)+i)
    end do
    call resize(a, size(a%p)-size(b))
  end subroutine aplt_remove
  subroutine amplt_remove(a,b)
    type(min_part_list_t), intent(inout) :: a
    integer :: b(:)
    integer i
    call assert(all(b(1:(size(b)-1))<b(2:)),'List is not ordered')
    call assert(b(size(b))<=size(a%p),&
         &'Tried to removed a nonexistant particle from a list')
    do i = size(b),1,-1
       a%p(b(i))=a%p(size(a%p)-size(b)+i)
    end do
    call resize(a, size(a%p)-size(b))
  end subroutine amplt_remove


  ! Return a single element or an array from the list and remove it
  ! if it is an array the array must be strictly increasing
  function splt_pop(a,b)
    type(particle_t) :: splt_pop
    type(part_list_t), intent(inout) :: a
    integer :: b
    splt_pop = a%p(b)
    call remove(a,b) ! Now remove the element
  end function splt_pop
  function smplt_pop(a,b)
    type(min_part_t) :: smplt_pop 
    type(min_part_list_t), intent(inout) :: a
    integer :: b
    smplt_pop = a%p(b)
    call remove(a,b) ! Now remove the element
  end function smplt_pop
  function aplt_pop(a,b)
    integer :: b(:)
    type(particle_t) :: aplt_pop(size(b))
    type(part_list_t), intent(inout) :: a
    aplt_pop = a%p(b)
    call remove(a,b) ! Now remove the element
  end function aplt_pop
  function amplt_pop(a,b)
    integer :: b(:)
    type(min_part_t) :: amplt_pop (size(b))
    type(min_part_list_t), intent(inout) :: a
    amplt_pop = a%p(b)
    call remove(a,b) ! Now remove the element
  end function amplt_pop
  function isfinite_p(p) result(r)
    type(particle_t), intent(in) :: p
    logical r
    r = isfinite(p%mp%r) .and. isfinite(p%mp%v) .and.  isfinite(p%f) &
         .and. isfinite(p%am) .and. isfinite(p%mp%w) 
    if (.not.r) then
       call error_halt('Something in the particle is infinite')    
    end if
  end function isfinite_p
  function isfinite_pa(p) result(r)
    type(particle_t), intent(in) :: p(:)
    logical r
    integer i
    r = isfinite(p%mp%r) .and. isfinite(p%mp%v) .and.  isfinite(p%f) &
         .and. isfinite(p%am) .and. isfinite(p%mp%w) 
    if (.not.r) then
       do i = 1,size(p)
          if (.not.isfinite(p(i)%mp%r)) then
             write(std_err,*)'Particle',i,'position is',p(i)%mp%r
          end if
          if (.not.isfinite(p(i)%mp%v)) then
             write(std_err,*)'Particle',i,'velocity is',p(i)%mp%v
          end if
          if (.not.isfinite(p(i)%f)) then
             write(std_err,*)'Particle',i,'force is',p(i)%f
          end if
          if (.not.isfinite(p(i)%am)) then
             write(std_err,*)'Particle',i,'angular moment is',p(i)%am
          end if
          if (.not.isfinite(p(i)%mp%w)) then
             write(std_err,*)'Particle',i,'angular velocity is',p(i)%mp%w
          end if
       end do
       call error_halt('Something in the particle is infinite')
    end if
  end function isfinite_pa
  subroutine energy_t_conv_to_si(energy)
    type(energy_t), intent(inout) :: energy
    call conv_energy_to_si(energy%lke)
    call conv_energy_to_si(energy%ake)
    call conv_energy_to_si(energy%gpe)
    call conv_energy_to_si(energy%cpe)
    call conv_energy_to_si(energy%tke)
    call conv_energy_to_si(energy%tpe)
    call conv_energy_to_si(energy%total)
  end subroutine energy_t_conv_to_si
  subroutine energy_t_conv_from_si(energy)
    type(energy_t), intent(inout) :: energy
    call conv_energy_from_si(energy%lke)
    call conv_energy_from_si(energy%ake)
    call conv_energy_from_si(energy%gpe)
    call conv_energy_from_si(energy%cpe)
    call conv_energy_from_si(energy%tke)
    call conv_energy_from_si(energy%tpe)
    call conv_energy_from_si(energy%total)
  end subroutine energy_t_conv_from_si

end module particle
