#include "units.h"
module program_units
  use precision
  use sysdep_io
  use vector
  implicit none
  !private SR2201BUG  

  ! Public functions
  public :: program_units_info  ! Display information about units
  public :: set_units           ! Set the program units
  public :: get_units           ! Get the program units
  public :: clear_units         ! Unset the program units
  public :: write_units         ! Write the units to a file pointer
  public :: read_units          ! Read the program units from a file


  public :: conv_time_to_si, conv_length_to_si, conv_mass_to_si
  public :: conv_velocity_to_si, conv_acceleration_to_si, conv_ang_vel_to_si
  public :: conv_force_to_si, conv_energy_to_si

  public :: conv_time_from_si, conv_length_from_si, conv_mass_from_si
  public :: conv_velocity_from_si, conv_acceleration_from_si, conv_ang_vel_from_si
  public :: conv_force_from_si, conv_energy_from_si

  ! Overloaded functions for special types and general types
  public :: conv_to_si, conv_from_si

  interface conv_to_si
     module procedure s_conv_general_to_si, aa_conv_general_to_si, &
          a_conv_general_to_si, & 
          v_conv_general_to_si, av_conv_general_to_si, & 
          srang_conv_general_to_si, arang_conv_general_to_si
  end interface
  interface conv_from_si
     module  procedure s_conv_general_from_si, &
          a_conv_general_from_si,aa_conv_general_from_si, & 
          v_conv_general_from_si, av_conv_general_from_si, & 
          srang_conv_general_from_si, arang_conv_general_from_si
  end interface

  interface conv_mass_to_si
     module procedure s_conv_mass_to_si, a_conv_mass_to_si
  end interface
  interface conv_mass_from_si
     module procedure s_conv_mass_from_si, a_conv_mass_from_si
  end interface

  interface conv_energy_to_si
     module procedure s_conv_energy_to_si, a_conv_energy_to_si
  end interface
  interface conv_energy_from_si
     module procedure s_conv_energy_from_si, a_conv_energy_from_si
  end interface

  interface conv_time_to_si
     module procedure s_conv_time_to_si, a_conv_time_to_si, &
          & sv_conv_time_to_si, av_conv_time_to_si
  end interface
  interface conv_time_from_si
     module procedure s_conv_time_from_si, a_conv_time_from_si, &
          & sv_conv_time_from_si, av_conv_time_from_si
  end interface

  interface conv_length_to_si
     module procedure s_conv_length_to_si, a_conv_length_to_si, &
          & sv_conv_length_to_si, av_conv_length_to_si
  end interface
  interface conv_length_from_si
     module procedure s_conv_length_from_si, a_conv_length_from_si, &
          &sv_conv_length_from_si, av_conv_length_from_si  
  end interface

  interface conv_velocity_to_si
     module procedure s_conv_vel_to_si, a_conv_vel_to_si,&
          & sv_conv_vel_to_si, av_conv_vel_to_si,&
          & srang_conv_vel_to_si, arang_conv_vel_to_si
  end interface
  interface conv_velocity_from_si
     module procedure s_conv_vel_from_si, a_conv_vel_from_si,&
          & sv_conv_vel_from_si, av_conv_vel_from_si,&
          & srang_conv_vel_from_si, arang_conv_vel_from_si
  end interface

  interface conv_ang_vel_to_si
     module procedure s_conv_ang_vel_to_si, a_conv_ang_vel_to_si,&
          & srang_conv_ang_vel_to_si, arang_conv_ang_vel_to_si
  end interface
  interface conv_ang_vel_from_si
     module procedure s_conv_ang_vel_from_si, a_conv_ang_vel_from_si,&
          & srang_conv_ang_vel_from_si, arang_conv_ang_vel_from_si
  end interface

  interface conv_acceleration_to_si
     module procedure s_conv_acceleration_to_si, a_conv_acceleration_to_si, &
          & sv_conv_acceleration_to_si, av_conv_acceleration_to_si
  end interface
  interface conv_acceleration_from_si
     module procedure &
          & s_conv_acceleration_from_si, a_conv_acceleration_from_si, &
          & sv_conv_acceleration_from_si, av_conv_acceleration_from_si
  end interface

  interface conv_force_to_si
     module procedure s_conv_force_to_si, a_conv_force_to_si,&
          & sv_conv_force_to_si, av_conv_force_to_si
  end interface
  interface conv_force_from_si
     module procedure s_conv_force_from_si, a_conv_force_from_si,&
          & sv_conv_force_from_si, av_conv_force_from_si
  end interface
  
  real(r_kind), parameter, public :: max_radius = MAX_RADIUS
  real(r_kind), parameter, public :: max_mass   = MAX_MASS
  real(r_kind), parameter, public :: dt         = DT

   ! Progam unit scale factors to SI units
  real(r_kind),private :: t_unit=-1 ! unit of time 
  real(r_kind),private :: l_unit=-1 ! unit of length
  real(r_kind),private :: m_unit=-1 ! unit of mass
  ! Derived units
  real(r_kind),private :: v_unit=-1 ! unit of vel
  real(r_kind),private :: a_unit=-1 ! unit of acceleration
  real(r_kind),private :: f_unit=-1 ! unit of mass
  real(r_kind),private :: e_unit=-1 ! unit of mass

  logical,private :: units_defined = .false.

  logical, private :: debug_units = .false.

  namelist /calc_units/ t_unit, l_unit, m_unit 
  private :: calc_units
contains
  subroutine program_units_info(fp)
    integer, intent(in) :: fp
    if (.not.units_defined) then
       write(fp,*) 'Units not defined'
    else
       write(fp,*) '***** SI units *****'
       write(fp,*) 'Length unit : ', l_unit
       write(fp,*) 'Time unit : ', t_unit
       write(fp,*) 'Mass unit : ', m_unit
    end if
    write(fp,*) '***** Program units *****'
    write(fp,*) 'Length unit MAX RADIUS : ', MAX_RADIUS
    write(fp,*) 'Mass unit MAX MASS : ', MAX_MASS
    write(fp,*) 'Time unit DT : ', DT
  end subroutine program_units_info
  subroutine clear_units
    units_defined = .false.
  end subroutine clear_units

  subroutine set_units(t,l,m)
    real(r_kind),intent(in) :: t
    real(r_kind),intent(in) :: l
    real(r_kind),intent(in) :: m

    if (debug_units) write(std_log,*)'set units :: t',t,'l',l,'m',m

    call assert(.not.units_defined,'Units already defined.')
    t_unit = t/DT          ! Set time unit
    l_unit = l/MAX_RADIUS  ! Length unit
    m_unit = m/MAX_MASS    ! Mass unit

    ! derived units
    v_unit = l_unit/t_unit             ! Vel unit
    a_unit = l_unit/t_unit**2          ! Acceleration unit
    f_unit = m_unit*a_unit             ! Force unit
    e_unit = f_unit*l_unit             ! Energy unit
    units_defined = .true.
  end subroutine set_units

  function get_units()
    real(r_kind) :: get_units(3)
    call assert(units_defined,'Units not defined.')
    get_units =(/t_unit,l_unit,m_unit/)
  end function get_units

  subroutine write_units(fp)
    integer, intent(in) :: fp
    call info_msg('Writing units.')
    write(fp, nml=calc_units)
  end subroutine write_units

  subroutine read_units(fp)
    integer, intent(in) :: fp
    call assert(.not.units_defined,'Units already defined.')
    call info_msg('Reading units.')
    read(fp, nml=calc_units)
    ! derived units
    v_unit = l_unit/t_unit             ! Vel unit
    a_unit = l_unit/t_unit**2          ! Acceleration unit
    f_unit = m_unit*a_unit             ! Force unit
    e_unit = f_unit*l_unit             ! Energy unit

    units_defined = .true.
  end subroutine read_units

  ! Mass units
  subroutine s_conv_mass_to_si(t)
    real(r_kind),intent(inout) :: t
    call assert(units_defined,'Units have not yet been defined')
    t = t*m_unit
  end subroutine s_conv_mass_to_si
  subroutine a_conv_mass_to_si(t)
    real(r_kind),intent(inout),dimension(:) :: t
    call assert(units_defined,'Units have not yet been defined')
    t = t*m_unit
  end subroutine a_conv_mass_to_si
  subroutine s_conv_mass_from_si(t)
    real(r_kind),intent(inout) :: t
    call assert(units_defined,'Units have not yet been defined')
    t = t/m_unit
  end subroutine s_conv_mass_from_si
  subroutine a_conv_mass_from_si(t)
    real(r_kind),intent(inout),dimension(:) :: t
    call assert(units_defined,'Units have not yet been defined')
    t = t/m_unit
  end subroutine a_conv_mass_from_si
  ! Energy units
  subroutine s_conv_energy_to_si(t)
    real(r_kind),intent(inout) :: t
    call assert(units_defined,'Units have not yet been defined')
    t = t*e_unit
  end subroutine s_conv_energy_to_si
  subroutine a_conv_energy_to_si(t)
    real(r_kind),intent(inout),dimension(:) :: t
    call assert(units_defined,'Units have not yet been defined')
    t = t*e_unit
  end subroutine a_conv_energy_to_si
  subroutine s_conv_energy_from_si(t)
    real(r_kind),intent(inout) :: t
    call assert(units_defined,'Units have not yet been defined')
    t = t/e_unit
  end subroutine s_conv_energy_from_si
  subroutine a_conv_energy_from_si(t)
    real(r_kind),intent(inout),dimension(:) :: t
    call assert(units_defined,'Units have not yet been defined')
    t = t/e_unit
  end subroutine a_conv_energy_from_si


  ! Time functions
  subroutine s_conv_time_to_si(t)
    real(r_kind),intent(inout) :: t
    call assert(units_defined,'Units have not yet been defined')
    t = t*t_unit
  end subroutine s_conv_time_to_si
  subroutine a_conv_time_to_si(t)
    real(r_kind),intent(inout),dimension(:) :: t
    call assert(units_defined,'Units have not yet been defined')
    t = t*t_unit
  end subroutine a_conv_time_to_si
  subroutine s_conv_time_from_si(t)
    real(r_kind),intent(inout) :: t
    call assert(units_defined,'Units have not yet been defined')
    t = t/t_unit
  end subroutine s_conv_time_from_si
  subroutine a_conv_time_from_si(t)
    real(r_kind),intent(inout),dimension(:) :: t
    call assert(units_defined,'Units have not yet been defined')
    t = t/t_unit
  end subroutine a_conv_time_from_si

  ! Length units
  subroutine s_conv_length_to_si(t)
    real(r_kind),intent(inout) :: t
    call assert(units_defined,'Units have not yet been defined')
    t = t*l_unit
  end subroutine s_conv_length_to_si
  subroutine a_conv_length_to_si(t)
    real(r_kind), intent(inout) :: t(:)
    call assert(units_defined,'Units have not yet been defined')
    t = t*l_unit
  end subroutine a_conv_length_to_si
  subroutine s_conv_length_from_si(t)
    real(r_kind),intent(inout) :: t
    call assert(units_defined,'Units have not yet been defined')
    t = t/l_unit
  end subroutine s_conv_length_from_si
  subroutine a_conv_length_from_si(t)
    real(r_kind),intent(inout),dimension(:) :: t
    call assert(units_defined,'Units have not yet been defined')
    t = t/l_unit
  end subroutine a_conv_length_from_si

  ! Vel units
  subroutine s_conv_vel_to_si(t)
    real(r_kind),intent(inout) :: t
    call assert(units_defined,'Units have not yet been defined')
    t = t*v_unit
  end subroutine s_conv_vel_to_si
  subroutine s_conv_vel_from_si(t)
    real(r_kind),intent(inout) :: t
    call assert(units_defined,'Units have not yet been defined')
    t = t/v_unit
  end subroutine s_conv_vel_from_si
  subroutine a_conv_vel_to_si(t)
    real(r_kind),intent(inout),dimension(:) :: t
    call assert(units_defined,'Units have not yet been defined')
    t = t*v_unit
  end subroutine a_conv_vel_to_si
  subroutine a_conv_vel_from_si(t)
    real(r_kind),intent(inout),dimension(:) :: t
    call assert(units_defined,'Units have not yet been defined')
    t = t/v_unit
  end subroutine a_conv_vel_from_si


  ! Acceleration units
  subroutine s_conv_acceleration_to_si(t)
    real(r_kind),intent(inout) :: t
    call assert(units_defined,'Units have not yet been defined')
    t = t*a_unit
  end subroutine s_conv_acceleration_to_si
  subroutine a_conv_acceleration_to_si(t)
    real(r_kind),intent(inout),dimension(:) :: t
    call assert(units_defined,'Units have not yet been defined')
    t = t*a_unit
  end subroutine a_conv_acceleration_to_si
  subroutine s_conv_acceleration_from_si(t)
    real(r_kind),intent(inout) :: t
    call assert(units_defined,'Units have not yet been defined')
    t = t/a_unit
  end subroutine s_conv_acceleration_from_si
  subroutine a_conv_acceleration_from_si(t)
    real(r_kind),intent(inout),dimension(:) :: t
    call assert(units_defined,'Units have not yet been defined')
    t = t/a_unit
  end subroutine a_conv_acceleration_from_si

  ! Force units
  subroutine s_conv_force_to_si(t)
    real(r_kind),intent(inout) :: t
    call assert(units_defined,'Units have not yet been defined')
    t = t*f_unit
  end subroutine s_conv_force_to_si
  subroutine a_conv_force_to_si(t)
    real(r_kind),intent(inout),dimension(:) :: t
    call assert(units_defined,'Units have not yet been defined')
    t = t*f_unit
  end subroutine a_conv_force_to_si
  subroutine s_conv_force_from_si(t)
    real(r_kind),intent(inout) :: t
    call assert(units_defined,'Units have not yet been defined')
    t = t/f_unit
  end subroutine s_conv_force_from_si
  subroutine a_conv_force_from_si(t)
    real(r_kind),intent(inout),dimension(:) :: t
    call assert(units_defined,'Units have not yet been defined')
    t = t/f_unit
  end subroutine a_conv_force_from_si

  ! Same again yawn yawn for vectors
  ! Time functions
  subroutine sv_conv_time_to_si(t)
    type(rvec_t),intent(inout) :: t
    call assert(units_defined,'Units have not yet been defined')
    t = t*t_unit
  end subroutine sv_conv_time_to_si
  subroutine av_conv_time_to_si(t)
    type(rvec_t),intent(inout),dimension(:) :: t
    call assert(units_defined,'Units have not yet been defined')
    t = t*t_unit
  end subroutine av_conv_time_to_si
  subroutine sv_conv_time_from_si(t)
    type(rvec_t),intent(inout) :: t
    call assert(units_defined,'Units have not yet been defined')
    t = t/t_unit
  end subroutine sv_conv_time_from_si
  subroutine av_conv_time_from_si(t)
    type(rvec_t),intent(inout),dimension(:) :: t
    call assert(units_defined,'Units have not yet been defined')
    t = t/t_unit
  end subroutine av_conv_time_from_si

  ! Length units
  subroutine sv_conv_length_to_si(t)
    type(rvec_t),intent(inout) :: t
    call assert(units_defined,'Units have not yet been defined')
    t = t*l_unit
  end subroutine sv_conv_length_to_si
  subroutine av_conv_length_to_si(t)
    type(rvec_t),intent(inout) :: t(:)
    call assert(units_defined,'Units have not yet been defined')
    t = t*l_unit
  end subroutine av_conv_length_to_si
  subroutine sv_conv_length_from_si(t)
    type(rvec_t),intent(inout) :: t
    call assert(units_defined,'Units have not yet been defined')
    t = t/l_unit
  end subroutine sv_conv_length_from_si
  subroutine av_conv_length_from_si(t)
    type(rvec_t),intent(inout),dimension(:) :: t
    call assert(units_defined,'Units have not yet been defined')
    t = t/l_unit
  end subroutine av_conv_length_from_si

  ! Vel units
  subroutine sv_conv_vel_to_si(t)
    type(rvec_t),intent(inout) :: t
    call assert(units_defined,'Units have not yet been defined')
    t = t*v_unit
  end subroutine sv_conv_vel_to_si
  subroutine av_conv_vel_to_si(t)
    type(rvec_t),intent(inout),dimension(:) :: t
    call assert(units_defined,'Units have not yet been defined')
    t = t*v_unit
  end subroutine av_conv_vel_to_si
  subroutine sv_conv_vel_from_si(t)
    type(rvec_t),intent(inout) :: t
    call assert(units_defined,'Units have not yet been defined')
    t = t/v_unit
  end subroutine sv_conv_vel_from_si
  subroutine av_conv_vel_from_si(t)
    type(rvec_t),intent(inout),dimension(:) :: t
    call assert(units_defined,'Units have not yet been defined')
    t = t/v_unit
  end subroutine av_conv_vel_from_si

  subroutine srang_conv_vel_to_si(t)
    type(rang_t),intent(inout) :: t
    call assert(units_defined,'Units have not yet been defined')
    t = t*v_unit
  end subroutine srang_conv_vel_to_si
  subroutine arang_conv_vel_to_si(t)
    type(rang_t),intent(inout),dimension(:) :: t
    call assert(units_defined,'Units have not yet been defined')
    t = t*v_unit
  end subroutine arang_conv_vel_to_si
  subroutine srang_conv_vel_from_si(t)
    type(rang_t),intent(inout) :: t
    call assert(units_defined,'Units have not yet been defined')
    t = t/v_unit
  end subroutine srang_conv_vel_from_si
  subroutine arang_conv_vel_from_si(t)
    type(rang_t),intent(inout),dimension(:) :: t
    call assert(units_defined,'Units have not yet been defined')
    t = t/v_unit
  end subroutine arang_conv_vel_from_si

  ! Angular vel units
  subroutine s_conv_ang_vel_to_si(t)
    real(r_kind),intent(inout) :: t
    call assert(units_defined,'Units have not yet been defined')
    t = t/t_unit
  end subroutine s_conv_ang_vel_to_si
  subroutine s_conv_ang_vel_from_si(t)
    real(r_kind),intent(inout) :: t
    call assert(units_defined,'Units have not yet been defined')
    t = t*t_unit
  end subroutine s_conv_ang_vel_from_si
  subroutine a_conv_ang_vel_to_si(t)
    real(r_kind),intent(inout),dimension(:) :: t
    call assert(units_defined,'Units have not yet been defined')
    t = t/t_unit
  end subroutine a_conv_ang_vel_to_si
  subroutine a_conv_ang_vel_from_si(t)
    real(r_kind),intent(inout),dimension(:) :: t
    call assert(units_defined,'Units have not yet been defined')
    t = t*t_unit
  end subroutine a_conv_ang_vel_from_si

  ! Angular vel units
  subroutine srang_conv_ang_vel_to_si(t)
    type(rang_t),intent(inout) :: t
    call assert(units_defined,'Units have not yet been defined')
    t = t/t_unit
  end subroutine srang_conv_ang_vel_to_si
  subroutine arang_conv_ang_vel_to_si(t)
    type(rang_t),intent(inout),dimension(:) :: t
    call assert(units_defined,'Units have not yet been defined')
    t = t/t_unit
  end subroutine arang_conv_ang_vel_to_si
  subroutine srang_conv_ang_vel_from_si(t)
    type(rang_t),intent(inout) :: t
    call assert(units_defined,'Units have not yet been defined')
    t = t*t_unit
  end subroutine srang_conv_ang_vel_from_si
  subroutine arang_conv_ang_vel_from_si(t)
    type(rang_t),intent(inout),dimension(:) :: t
    call assert(units_defined,'Units have not yet been defined')
    t = t*t_unit
  end subroutine arang_conv_ang_vel_from_si

  ! Acceleration units
  subroutine sv_conv_acceleration_to_si(t)
    type(rvec_t),intent(inout) :: t
    call assert(units_defined,'Units have not yet been defined')
    t = t*a_unit
  end subroutine sv_conv_acceleration_to_si
  subroutine av_conv_acceleration_to_si(t)
    type(rvec_t),intent(inout),dimension(:) :: t
    call assert(units_defined,'Units have not yet been defined')
    t = t*a_unit
  end subroutine av_conv_acceleration_to_si
  subroutine sv_conv_acceleration_from_si(t)
    type(rvec_t),intent(inout) :: t
    call assert(units_defined,'Units have not yet been defined')
    t = t/a_unit
  end subroutine sv_conv_acceleration_from_si
  subroutine av_conv_acceleration_from_si(t)
    type(rvec_t),intent(inout),dimension(:) :: t
    call assert(units_defined,'Units have not yet been defined')
    t = t/a_unit
  end subroutine av_conv_acceleration_from_si

  ! Force units
  subroutine sv_conv_force_to_si(t)
    type(rvec_t),intent(inout) :: t
    call assert(units_defined,'Units have not yet been defined')
    t = t*f_unit
  end subroutine sv_conv_force_to_si
  subroutine av_conv_force_to_si(t)
    type(rvec_t),intent(inout),dimension(:) :: t
    call assert(units_defined,'Units have not yet been defined')
    t = t*f_unit
  end subroutine av_conv_force_to_si
  subroutine sv_conv_force_from_si(t)
    type(rvec_t),intent(inout) :: t
    call assert(units_defined,'Units have not yet been defined')
    t = t/f_unit
  end subroutine sv_conv_force_from_si
  subroutine av_conv_force_from_si(t)
    type(rvec_t),intent(inout),dimension(:) :: t
    call assert(units_defined,'Units have not yet been defined')
    t = t/f_unit
  end subroutine av_conv_force_from_si



  subroutine s_conv_general_from_si(x,T,L,M)
    real(r_kind), intent(inout) :: x
    integer, intent(in), optional :: T,L,M
    real(r_kind) :: f 
    f = 1
    call assert(units_defined,'Units have not yet been defined')
    if (present(T)) f = f * t_unit**T
    if (present(L)) f = f * l_unit**L
    if (present(M)) f = f * m_unit**M
    x = x / f
  end subroutine s_conv_general_from_si
  subroutine a_conv_general_from_si(x,T,L,M)
    real(r_kind), intent(inout) :: x(:)
    integer, intent(in), optional :: T,L,M
    real(r_kind) :: f 
    f = 1
    call assert(units_defined,'Units have not yet been defined')
    if (present(T)) f = f * t_unit**T
    if (present(L)) f = f * l_unit**L
    if (present(M)) f = f * m_unit**M
    x = x / f
  end subroutine a_conv_general_from_si
  subroutine aa_conv_general_from_si(x,T,L,M)
    real(r_kind), intent(inout) :: x(:,:)
    integer, intent(in), optional :: T,L,M
    real(r_kind) :: f 
    f = 1
    call assert(units_defined,'Units have not yet been defined')
    if (present(T)) f = f * t_unit**T
    if (present(L)) f = f * l_unit**L
    if (present(M)) f = f * m_unit**M
    x = x / f
  end subroutine aa_conv_general_from_si
  subroutine v_conv_general_from_si(x,T,L,M)
    type(rvec_t), intent(inout) :: x
    integer, intent(in), optional :: T,L,M
    real(r_kind) :: f 
    f = 1
    call assert(units_defined,'Units have not yet been defined')
    if (present(T)) f = f * t_unit**T
    if (present(L)) f = f * l_unit**L
    if (present(M)) f = f * m_unit**M
    x = x / f
  end subroutine v_conv_general_from_si
  subroutine av_conv_general_from_si(x,T,L,M)
    type(rvec_t), intent(inout) :: x(:)
    integer, intent(in), optional :: T,L,M
    real(r_kind) :: f 
    f = 1
    call assert(units_defined,'Units have not yet been defined')
    if (present(T)) f = f * t_unit**T
    if (present(L)) f = f * l_unit**L
    if (present(M)) f = f * m_unit**M
    x = x / f
  end subroutine av_conv_general_from_si
  subroutine srang_conv_general_from_si(x,T,L,M)
    type(rang_t), intent(inout) :: x
    integer, intent(in), optional :: T,L,M
    real(r_kind) :: f 
    f = 1
    call assert(units_defined,'Units have not yet been defined')
    if (present(T)) f = f * t_unit**T
    if (present(L)) f = f * l_unit**L
    if (present(M)) f = f * m_unit**M
    x = x / f
  end subroutine srang_conv_general_from_si
  subroutine arang_conv_general_from_si(x,T,L,M)
    type(rang_t), intent(inout) :: x(:)
    integer, intent(in), optional :: T,L,M
    real(r_kind) :: f 
    f = 1
    call assert(units_defined,'Units have not yet been defined')
    if (present(T)) f = f * t_unit**T
    if (present(L)) f = f * l_unit**L
    if (present(M)) f = f * m_unit**M
    x = x / f
  end subroutine arang_conv_general_from_si

  subroutine s_conv_general_to_si(x,T,L,M)
    real(r_kind), intent(inout) :: x
    integer, intent(in), optional :: T,L,M
    real(r_kind) :: f 
    call assert(units_defined,'Units have not yet been defined')
    f = 1
    if (present(T)) f = f * t_unit**T
    if (present(L)) f = f * l_unit**L
    if (present(M)) f = f * m_unit**M
    x = f*x
  end subroutine s_conv_general_to_si
  subroutine a_conv_general_to_si(x,T,L,M)
    real(r_kind), intent(inout) :: x(:)
    integer, intent(in), optional :: T,L,M
    real(r_kind) :: f
    call assert(units_defined,'Units have not yet been defined')
    f = 1
    if (present(T)) f = f * t_unit**T
    if (present(L)) f = f * l_unit**L
    if (present(M)) f = f * m_unit**M
    x = f*x
  end subroutine a_conv_general_to_si
  subroutine aa_conv_general_to_si(x,T,L,M)
    real(r_kind), intent(inout) :: x(:,:)
    integer, intent(in), optional :: T,L,M
    real(r_kind) :: f
    call assert(units_defined,'Units have not yet been defined')
    f = 1
    if (present(T)) f = f * t_unit**T
    if (present(L)) f = f * l_unit**L
    if (present(M)) f = f * m_unit**M
    x = f*x
  end subroutine aa_conv_general_to_si
  subroutine v_conv_general_to_si(x,T,L,M)
    type(rvec_t), intent(inout) :: x
    integer, intent(in), optional :: T,L,M
    real(r_kind) :: f
    call assert(units_defined,'Units have not yet been defined')
    f = 1
    if (present(T)) f = f * t_unit**T
    if (present(L)) f = f * l_unit**L
    if (present(M)) f = f * m_unit**M
    x = f*x
  end subroutine v_conv_general_to_si
  subroutine av_conv_general_to_si(x,T,L,M)
    type(rvec_t), intent(inout) :: x(:)
    integer, intent(in), optional :: T,L,M
    real(r_kind) :: f 
    call assert(units_defined,'Units have not yet been defined')
    f = 1
    if (present(T)) f = f * t_unit**T
    if (present(L)) f = f * l_unit**L
    if (present(M)) f = f * m_unit**M
    x = f*x
  end subroutine av_conv_general_to_si
  subroutine srang_conv_general_to_si(x,T,L,M)
    type(rang_t), intent(inout) :: x
    integer, intent(in), optional :: T,L,M
    real(r_kind) :: f
    call assert(units_defined,'Units have not yet been defined')
    f = 1
    if (present(T)) f = f * t_unit**T
    if (present(L)) f = f * l_unit**L
    if (present(M)) f = f * m_unit**M
    x = f*x
  end subroutine srang_conv_general_to_si
  subroutine arang_conv_general_to_si(x,T,L,M)
    type(rang_t), intent(inout) :: x(:)
    integer, intent(in), optional :: T,L,M
    real(r_kind) :: f 
    call assert(units_defined,'Units have not yet been defined')
    f = 1
    if (present(T)) f = f * t_unit**T
    if (present(L)) f = f * l_unit**L
    if (present(M)) f = f * m_unit**M
    x = f*x
  end subroutine arang_conv_general_to_si
end  module program_units
