! Determines how long the program runs for 
! if any criteria is satisfied it terminates
! these criteria are checked every program step
! if the program time is greater than  max_t 
! if more than max_steps program steps have occured
!
! These criteria are checked every check_t seconds or check_s steps
! after min_t or min_steps have occured
! if the program has been running for more than max_r_t seconds
! if the mean speed is less than mean_speed
! if the max speed is less than max_speed 
! nothing in this module gets called on any except the root node 

module termination 
  use precision
  use sysdep_io
  use vector
  use program_units
  use particle
  use pro_state_f90_def
  implicit none
  !private SR2201BUG

  public :: read_termination_param   ! Read namelist specifying termination criterion
  public :: termination_criterion    ! Decide whether to terminate
  
  real(r_kind) :: max_t=big_real
  integer      :: max_s=HUGE(max_s)
  real(r_kind) :: max_rt=big_real
  
  real(r_kind) :: check_t=big_real
  real(r_kind) :: check_s=big_real

  real(r_kind) :: min_t=-big_real
  integer      :: min_s=-HUGE(min_s)
  real(r_kind) :: mean_speed=-big_real
  real(r_kind) :: max_speed=-big_real
  
  real(r_kind) :: count_t
  real(r_kind) :: count_s
    
  namelist /termination_param/ check_t,check_s,max_t,max_s,max_rt,&
       min_t,min_s,mean_speed,max_speed
contains
  subroutine read_termination_param(fp)
    integer, intent(in) :: fp
    read(fp, nml=termination_param)
    call conv_velocity_from_si(mean_speed)
    call conv_velocity_from_si(max_speed)
    count_t = check_t
    count_s = check_s
  end subroutine read_termination_param
  function termination_criterion(p,s,t) result(tc)
    type(particle_t), intent(in)      :: p(:)
    real(r_kind), intent(in)      :: t
    integer, intent(in)           :: s
 
    logical                       :: tc
    real(r_kind) max_v
    real(r_kind) mean_v
   
    tc = .false.
     
    if (s >= max_s) then
       write(std_log,*)"Max steps termination criterion satisfied"
       tc = .true.
    else if (t >= max_t) then
       write(std_log,*)"Max time termination criterion satisfied"
       tc = .true.
    else if ((s > min_s) .and.  (t>min_t)) then
       if ((count_s<s) .or. (count_t<t)) then
          call mean_and_max(mean_v, max_v, length(p%mp%v))
          count_s = s + check_s
          count_t = t + check_t
          if (mean_v < mean_speed) then
             write(std_log,*)"Mean speed termination criterion satisfied"
             tc = .true.
          end if
          if (max_v < max_speed) then
             write(std_log,*)"Max speed termination criterion satisfied"
             tc = .true.
          end if
       end if
    end if
  end function termination_criterion
  subroutine mean_and_max(mean_v, max_v, v)
    real(r_kind), intent(out) :: mean_v, max_v
    real(r_kind), intent(in) :: v(:) 
    max_v = maxval(v)
    if (size(v)>0) then
      mean_v =  sum(v)/size(v)
   else
      mean_v = 0
   end if
  end subroutine mean_and_max
end module termination
