! Reads two filename from standard input 
! Then compares the data in the two file names and writes a 
! report to standard output
! Use double precision
! Read one line at a time since we don't how many there are
program comp_dem
  implicit none


  integer, parameter :: s_len = 100 ! maximum length for strings or lines of data
  integer, parameter :: r_kind = kind(0.d0)  !-- real double precision

  character(len=s_len) :: n1,n2
  real(r_kind),allocatable :: t1(:), t2(:)
  integer,allocatable :: s1(:), s2(:)
  real(r_kind), allocatable :: d1(:,:,:),d2(:,:,:),d(:) ! Compares vector data
  character(len=s_len) :: buf  ! Buffer for reading in lines of data
  integer :: fp1=8,fp2=9
  integer :: nl ! Number of lines
  integer nt,np,n
  real(r_kind) :: maxe,var
  read(*,'(A, A)')buf  ! read the file names from standard input
  call get_token(buf,n1) ! get a space delimited token
  call get_token(buf,n2)
  fp1 = my_open(n1)
  fp2 = my_open(n2)

  nl = 0
  nt = 0
  do
     read(fp1,'(A)',end=100)buf     ! Read in a whole line
     nl = nl+1
     if (index(buf,'step')/=0) nt = nt+1
  end do
100 np = nl/nt-1
  rewind fp1

  !  write(*,*)'Read',nl,'lines.'
  !  write(*,*)'There are ',nt,' time steps.'
  !  write(*,*)'There are ',np,' particles.'
  allocate(t1(nt),t2(nt),s1(nt),s2(nt),d1(4,np,nt),d2(4,np,nt),d(np*nt))
  ! Step through reading all the data
  do n=1,nt
     read(fp1,'(A)')buf
     t1(n) = read_string(buf,'time')
     read(fp1,*)d1(:,:,n)
     read(fp2,'(A)')buf
     t2(n) = read_string(buf,'time')
     read(fp2,*)d2(:,:,n)
  end do

  d1 =  (d2-d1)**2
  d = reshape(sum(d1(2:4,:,:),1),(/np*nt/))

  maxe = sqrt(maxval(d))
  var = sqrt(sum(d1))
  write(*,*)'Comparing files ',trim(n1),' and ',trim(n2)
  if (maxe /= 0) then
     write(*,'(A,F14.8)')'Max error is ',maxe
  end if
  write(*,'(A,F14.8)')'Variance is ', var
  write(*,'(A,F14.8)')'Max time difference is ', maxval(abs(t1-t2))
contains   
  subroutine get_token(in,out)
    character(len=*), intent(inout) :: in
    character(len=*), intent(out) :: out
    integer n
    in = adjustl(in) ! Remove leading blanks
    n = scan(in,' ')
    if (n==0) then
       out = in
       in = ' '
    else
       out = in(1:n) // repeat(' ',len(out)-n)
       in = in((n+1):) // repeat(' ',n)
    end if
  end subroutine get_token
  subroutine assign_lun (lun)     !Return a free unit number
    integer, intent(out) :: lun 
    integer :: iostat
    logical :: used
    integer, parameter :: min_lun = 10
    integer, parameter :: max_lun = 99
    do lun = min_lun , max_lun
       inquire(unit=lun, opened=used, iostat=iostat)
       if (iostat == 0 .and. .not.used) return
    end do
    write(*,*) 'Failed to assign lun'
    stop
  end subroutine assign_lun

  function my_open(n) result(fp)
    character(len=*), intent(in) :: n
    integer :: fp
    call assign_lun(fp)
    open(fp,file=n,status='old',action='read',err=99)
    return
99  write(*,*) 'Failed to open file ',n
    stop
  end function my_open

  ! return the value of a number follwing a string
  function read_string(str,substr) result(r)
    character(len=*), intent(in) :: str
    character(len=*), intent(in) :: substr
    real(r_kind) r
    integer p
    p = index(str,substr)
    read(str(p+len(substr):),*)r
  end function read_string
  subroutine error(s)
    character(len=*), intent(in) :: s
    write(*,*)'Error ',s
    stop
  end subroutine error
end program comp_dem
